import errno
import socket
import time

import pytest
from pr2test.context_manager import skip_if_not_supported
from pr2test.marks import require_root
from utils import require_kernel

from pyroute2 import IPRoute, NetlinkError
from pyroute2.common import AF_MPLS
from pyroute2.netlink.rtnl.rtmsg import RTNH_F_ONLINK

pytestmark = [require_root()]


def test_route_get_target_strict_check(context):
    if not context.ipr.get_default_routes(table=254):
        pytest.skip('no default IPv4 routes')
    require_kernel(4, 20)
    with IPRoute(strict_check=True) as ip:
        rts = ip.get_routes(family=socket.AF_INET, dst='8.8.8.8', table=254)
        assert len(rts) > 0


def test_extended_error_on_route(context):
    require_kernel(4, 20)
    # specific flags, cannot use context.ip
    with IPRoute(ext_ack=True, strict_check=True) as ip:
        with pytest.raises(NetlinkError) as e:
            ip.route("get", dst="1.2.3.4", table=254, dst_len=0)
    assert abs(e.value.code) == errno.EINVAL
    # on 5.10 kernel, full message is 'ipv4: rtm_src_len and
    # rtm_dst_len must be 32 for IPv4'
    assert "rtm_dst_len" in str(e.value)


@pytest.mark.parametrize(
    'proto', (('static', 'boot'), (4, 3), ('boot', 4), (3, 'static'))
)
def test_route_proto(context, proto):

    proto, fake = proto
    ipaddr = context.new_ipaddr
    gateway = context.new_ipaddr
    ipnet = context.new_ip4net
    ifname = context.new_ifname

    (
        context.ndb.interfaces.create(ifname=ifname, kind='dummy', state='up')
        .add_ip(f'{ipaddr}/24')
        .commit()
    )
    spec = {
        'dst': ipnet.network,
        'dst_len': ipnet.netmask,
        'gateway': gateway,
        'proto': proto,
    }
    context.ndb.routes.create(**spec).commit()

    with pytest.raises(NetlinkError):
        context.ipr.route(
            'del',
            dst=f'{ipnet.network}/{ipnet.netmask}',
            gateway=f'{gateway}',
            proto=fake,
        )
    context.ipr.route(
        'del',
        dst=f'{ipnet.network}/{ipnet.netmask}',
        gateway=f'{gateway}',
        proto=proto,
    )


def test_route_oif_as_iterable(context):
    index, ifname = context.default_interface
    ipnet = context.new_ip4net
    spec = {'dst': ipnet.network, 'dst_len': ipnet.netmask, 'oif': (index,)}
    context.ndb.interfaces[ifname].set('state', 'up').commit()
    context.ipr.route('add', **spec)
    route = context.ndb.routes.wait(
        dst=ipnet.network, dst_len=ipnet.netmask, timeout=5
    )
    assert route['oif'] == index


def test_route_get_target(context):
    if not context.ipr.get_default_routes(table=254):
        pytest.skip('no default IPv4 routes')
    rts = context.ipr.get_routes(
        family=socket.AF_INET, dst='8.8.8.8', table=254
    )
    assert len(rts) > 0


def test_route_get_target_default_ipv4(context):
    rts = context.ipr.get_routes(dst='127.0.0.1')
    assert len(rts) > 0


def test_route_get_target_default_ipv6(context):
    rts = context.ipr.get_routes(dst='::1')
    assert len(rts) > 0


@skip_if_not_supported
@pytest.mark.parametrize('family', (socket.AF_INET, socket.AF_INET6))
def test_route_mpls_via(context, family):
    if family == socket.AF_INET:
        address = context.new_ipaddr
    else:
        ip6net = context.new_ip6net
        address = str(ip6net.network) + '7c32'
    label = 0x20
    index, ifname = context.default_interface
    context.ndb.interfaces[ifname].set('state', 'up').commit()
    context.ipr.route(
        'add',
        **{
            'family': AF_MPLS,
            'oif': index,
            'via': {'family': family, 'addr': address},
            'newdst': {'label': label, 'bos': 1},
        },
    )

    rt = context.ipr.get_routes(oif=index, family=AF_MPLS)[0]
    assert rt.get_attr('RTA_VIA')['addr'] == address
    assert rt.get_attr('RTA_VIA')['family'] == family
    assert rt.get_attr('RTA_NEWDST')[0]['label'] == label
    assert len(rt.get_attr('RTA_NEWDST')) == 1
    context.ipr.route(
        'del',
        **{
            'family': AF_MPLS,
            'oif': index,
            'dst': {'label': 0x10, 'bos': 1},
            'via': {'family': family, 'addr': address},
            'newdst': {'label': label, 'bos': 1},
        },
    )
    assert len(context.ipr.get_routes(oif=index, family=AF_MPLS)) == 0


@skip_if_not_supported
@pytest.mark.parametrize(
    'newdst', ({'label': 0x21, 'bos': 1}, [{'label': 0x21, 'bos': 1}])
)
def test_route_mpls_swap_newdst(context, newdst):
    index, _ = context.default_interface
    req = {
        'family': AF_MPLS,
        'oif': index,
        'dst': {'label': 0x20, 'bos': 1},
        'newdst': newdst,
    }
    context.ipr.route('add', **req)

    rt = context.ipr.get_routes(oif=index, family=AF_MPLS)[0]
    assert rt.get_attr('RTA_DST')[0]['label'] == 0x20
    assert len(rt.get_attr('RTA_DST')) == 1
    assert rt.get_attr('RTA_NEWDST')[0]['label'] == 0x21
    assert len(rt.get_attr('RTA_NEWDST')) == 1
    context.ipr.route('del', **req)
    assert len(context.ipr.get_routes(oif=index, family=AF_MPLS)) == 0


@pytest.mark.parametrize('mode', ('normal', 'raw'))
def test_route_multipath(context, mode):
    index, ifname = context.default_interface
    ipaddr = context.new_ipaddr
    gateway1 = context.new_ipaddr
    gateway2 = context.new_ipaddr
    ip4net = context.new_ip4net

    if mode == 'normal':
        multipath = [{'gateway': gateway1}, {'gateway': gateway2}]
    elif mode == 'raw':
        multipath = [
            {'hops': 20, 'oif': index, 'attrs': [['RTA_GATEWAY', gateway1]]},
            {'hops': 30, 'oif': index, 'attrs': [['RTA_GATEWAY', gateway2]]},
        ]

    context.ndb.interfaces[ifname].add_ip(f'{ipaddr}/24').commit()
    context.ipr.route(
        'add', dst=ip4net.network, dst_len=ip4net.netmask, multipath=multipath
    )

    route = context.ndb.routes.wait(
        dst=ip4net.network, dst_len=ip4net.netmask, timeout=5
    )
    nh1 = route['multipath'][0]
    nh2 = route['multipath'][1]

    assert nh1['gateway'] == gateway1
    assert nh2['gateway'] == gateway2


@pytest.mark.parametrize('flags', (RTNH_F_ONLINK, ['onlink']))
def test_route_onlink(context, flags):
    ip4net = context.new_ip4net
    ipaddr = context.new_ipaddr
    index, ifname = context.default_interface

    context.ipr.route(
        'add',
        dst=ip4net.network,
        dst_len=ip4net.netmask,
        gateway=ipaddr,
        oif=index,
        flags=flags,
    )
    route = context.ndb.routes.wait(
        dst=ip4net.network, dst_len=ip4net.netmask, timeout=5
    )
    assert route['oif'] == index
    route.remove().commit()


@pytest.mark.parametrize('flags', (RTNH_F_ONLINK, ['onlink']))
def test_route_onlink_multipath(context, flags):
    ip4net = context.new_ip4net
    gateway1 = context.new_ipaddr
    gateway2 = context.new_ipaddr
    index, ifname = context.default_interface
    context.ipr.route(
        'add',
        dst=ip4net.network,
        dst_len=ip4net.netmask,
        multipath=[
            {'gateway': gateway1, 'oif': 1, 'flags': flags},
            {'gateway': gateway2, 'oif': 1, 'flags': flags},
        ],
    )

    route = context.ndb.routes.wait(
        dst=ip4net.network, dst_len=ip4net.netmask, timeout=5
    )
    nh1 = route['multipath'][0]
    nh2 = route['multipath'][1]

    assert nh1['gateway'] == gateway1
    assert nh2['gateway'] == gateway2

    route.remove().commit()


@skip_if_not_supported
def _test_lwtunnel_multipath_mpls(context):
    ip4net = context.new_ip4net
    index, ifname = context.default_interface
    gateway = context.new_ipaddr
    ipaddr = context.new_ipaddr

    context.ndb.interfaces[ifname].add_ip(f'{ipaddr}/24').commit()

    context.ipr.route(
        'add',
        dst=f'{ip4net.network}/{ip4net.netmask}',
        multipath=[
            {'encap': {'type': 'mpls', 'labels': 500}, 'oif': index},
            {
                'encap': {'type': 'mpls', 'labels': '600/700'},
                'gateway': gateway,
            },
        ],
    )

    routes = context.ipr.route(
        'dump', dst=ip4net.network, dst_len=ip4net.netmask
    )
    assert len(routes) == 1
    mp = routes[0].get_attr('RTA_MULTIPATH')
    assert len(mp) == 2
    assert mp[0]['oif'] == 1
    assert mp[0].get_attr('RTA_ENCAP_TYPE') == 1
    labels = mp[0].get_attr('RTA_ENCAP').get_attr('MPLS_IPTUNNEL_DST')
    assert len(labels) == 1
    assert labels[0]['bos'] == 1
    assert labels[0]['label'] == 500
    assert mp[1].get_attr('RTA_ENCAP_TYPE') == 1
    labels = mp[1].get_attr('RTA_ENCAP').get_attr('MPLS_IPTUNNEL_DST')
    assert len(labels) == 2
    assert labels[0]['bos'] == 0
    assert labels[0]['label'] == 600
    assert labels[1]['bos'] == 1
    assert labels[1]['label'] == 700


@skip_if_not_supported
@pytest.mark.parametrize(
    'lid,lnum,labels',
    (
        ('list+dict', 2, [{'bos': 0, 'label': 226}, {'bos': 1, 'label': 227}]),
        ('list+int', 2, [226, 227]),
        ('str', 2, '226/227'),
        ('list+dict', 1, [{'bos': 1, 'label': 227}]),
        ('list+int', 1, [227]),
        ('str', 1, '227'),
        ('dict', 1, {'bos': 1, 'label': 227}),
        ('int', 1, 227),
    ),
)
def test_lwtunnel_mpls_labels(context, lid, lnum, labels):
    ip4net = context.new_ip4net
    ipaddr = context.new_ipaddr
    gateway = context.new_ipaddr
    index, ifname = context.default_interface
    context.ndb.interfaces[ifname].add_ip(f'{ipaddr}/24').commit()
    context.ipr.route(
        'add',
        dst=ip4net.network,
        dst_len=ip4net.netmask,
        encap={'type': 'mpls', 'labels': labels},
        gateway=gateway,
    )
    routes = context.ipr.route(
        'dump', dst=ip4net.network, dst_len=ip4net.netmask
    )
    assert len(routes) == 1
    route = routes[0]
    assert route.get_attr('RTA_ENCAP_TYPE') == 1
    assert route.get_attr('RTA_GATEWAY') == gateway
    labels = route.get_attr('RTA_ENCAP').get_attr('MPLS_IPTUNNEL_DST')
    assert len(labels) == lnum
    if lnum == 2:
        assert labels[0]['bos'] == 0
        assert labels[0]['label'] == 226
        assert labels[1]['bos'] == 1
        assert labels[1]['label'] == 227
    else:
        assert labels[0]['bos'] == 1
        assert labels[0]['label'] == 227

    context.ipr.route('del', dst=f'{ip4net.network}/{ip4net.netmask}')


def test_route_change_existing(context):

    index, ifname = context.default_interface
    ipaddr = context.new_ipaddr
    gateway1 = context.new_ipaddr
    gateway2 = context.new_ipaddr
    ip4net = context.new_ip4net

    context.ndb.interfaces[ifname].add_ip(f'{ipaddr}/24').commit()
    context.ipr.route(
        'add', dst=ip4net.network, dst_len=ip4net.netmask, gateway=gateway1
    )
    context.ndb.routes.wait(
        dst=ip4net.network, dst_len=ip4net.netmask, gateway=gateway1, timeout=5
    )

    context.ipr.route(
        'change', dst=ip4net.network, dst_len=ip4net.netmask, gateway=gateway2
    )
    context.ndb.routes.wait(
        dst=ip4net.network, dst_len=ip4net.netmask, gateway=gateway2, timeout=5
    )


def test_route_change_not_existing_fail(context):
    # route('change', ...) should fail, if no route exists
    index, ifname = context.default_interface
    ipaddr = context.new_ipaddr
    gateway2 = context.new_ipaddr
    ip4net = context.new_ip4net

    context.ndb.interfaces[ifname].add_ip(f'{ipaddr}/24').commit()

    with pytest.raises(NetlinkError) as e:
        context.ipr.route(
            'change',
            dst=ip4net.network,
            dst_len=ip4net.netmask,
            gateway=gateway2,
        )
    assert e.value.code == errno.ENOENT


def test_route_replace_existing(context):
    # route('replace', ...) should succeed, if route exists
    index, ifname = context.default_interface
    ipaddr = context.new_ipaddr
    gateway1 = context.new_ipaddr
    gateway2 = context.new_ipaddr
    ip4net = context.new_ip4net

    context.ndb.interfaces[ifname].add_ip(f'{ipaddr}/24').commit()
    context.ipr.route(
        'add', dst=ip4net.network, dst_len=ip4net.netmask, gateway=gateway1
    )
    context.ndb.routes.wait(
        dst=ip4net.network, dst_len=ip4net.netmask, gateway=gateway1, timeout=5
    )

    context.ipr.route(
        'replace', dst=ip4net.network, dst_len=ip4net.netmask, gateway=gateway2
    )
    context.ndb.routes.wait(
        dst=ip4net.network, dst_len=ip4net.netmask, gateway=gateway2, timeout=5
    )


def test_route_replace_not_existing(context):
    # route('replace', ...) should succeed, if route doesn't exist
    index, ifname = context.default_interface
    ipaddr = context.new_ipaddr
    gateway2 = context.new_ipaddr
    ip4net = context.new_ip4net

    context.ndb.interfaces[ifname].add_ip(f'{ipaddr}/24').commit()
    context.ipr.route(
        'replace', dst=ip4net.network, dst_len=ip4net.netmask, gateway=gateway2
    )
    context.ndb.routes.wait(
        dst=ip4net.network, dst_len=ip4net.netmask, gateway=gateway2, timeout=5
    )


def test_flush_routes(context):

    index, ifname = context.default_interface
    ipaddr = context.new_ipaddr
    gateway = context.new_ipaddr

    context.ndb.interfaces[ifname].add_ip(f'{ipaddr}/24').commit()
    for net in [context.new_ip4net for _ in range(10)]:
        context.ipr.route(
            'add',
            dst=net.network,
            dst_len=net.netmask,
            gateway=gateway,
            table=10101,
            oif=index,
        )
        context.ndb.routes.wait(
            dst=net.network, dst_len=net.netmask, table=10101, timeout=5
        )

    assert context.ndb.routes.summary().filter(table=10101).count() == 10
    context.ipr.flush_routes(table=10101, family=socket.AF_INET6)
    assert context.ndb.routes.summary().filter(table=10101).count() == 10
    context.ipr.flush_routes(table=10101, family=socket.AF_INET)
    for _ in range(5):
        if context.ndb.routes.summary().filter(table=10101).count() == 0:
            break
        time.sleep(0.1)
    else:
        raise Exception('route table not flushed')



import atexit
import errno
import gc
import getpass
import os
import resource

import pytest

from pyroute2 import NDB, IPRoute, NetNS
from pyroute2.common import uifname

RESPAWNS = 200
pytestmark = [
    pytest.mark.skipif(getpass.getuser() != 'root', reason='no root access')
]


@pytest.fixture
def fds():
    soft, hard = resource.getrlimit(resource.RLIMIT_NOFILE)
    try:
        resource.setrlimit(
            resource.RLIMIT_NOFILE,
            (min(soft, RESPAWNS // 2), min(hard, RESPAWNS // 2)),
        )
    except ValueError:
        pytest.skip('cannot set RLIMIT_NOFILE')
    fds_before = os.listdir(f'/proc/{os.getpid()}/fd/')
    yield fds_before
    gc.collect()
    fds_after = os.listdir(f'/proc/{os.getpid()}/fd/')
    assert len(fds_after) <= len(fds_before)


def test_respawn_iproute_sync(fds):
    for _ in range(RESPAWNS):
        with IPRoute() as i:
            i.bind()
            i.link_lookup(ifname='lo')


def test_respawn_iproute_async(fds):
    for _ in range(RESPAWNS):
        with IPRoute() as i:
            i.bind(async_cache=True)
            i.link_lookup(ifname='lo')


def test_respawn_ndb(fds):
    for _ in range(RESPAWNS):
        with NDB() as i:
            assert i.interfaces.count() > 0
            assert i.addresses.count() > 0
            assert i.routes.count() > 0
            assert i.neighbours.count() > 0


def test_bridge_fd_leaks(fds):
    ifs = []
    for _ in range(RESPAWNS):
        ifs.append(uifname())
    with NDB() as ndb:
        for name in ifs:
            ndb.interfaces.create(ifname=name, kind='bridge').apply()
    with NDB() as ndb:
        for name in ifs:
            ndb.interfaces[name].remove().apply()


def test_tuntap_fd_leaks(fds):
    ifs = []
    for _ in range(RESPAWNS):
        ifs.append(uifname())
    with NDB() as ndb:
        for name in ifs:
            ndb.interfaces.create(
                ifname=name, kind='tuntap', mode='tun'
            ).apply()
    with NDB() as ndb:
        for name in ifs:
            ndb.interfaces[name].remove().apply()


def test_fd_leaks(fds):
    for i in range(RESPAWNS):
        nsid = 'leak_%i' % i
        ns = NetNS(nsid)
        ns.close()
        ns.remove()
        if hasattr(atexit, '_exithandlers'):
            assert ns.close not in atexit._exithandlers


def test_fd_leaks_nonexistent_ns(fds):
    for i in range(RESPAWNS):
        nsid = 'non_existent_leak_%i' % i
        try:
            with NetNS(nsid, flags=0):
                pass
        except OSError as e:
            assert e.errno in (errno.ENOENT, errno.EPIPE)



import logging
import uuid

import pytest
from pr2test.context_manager import make_test_matrix
from pr2test.marks import require_root
from pr2test.tools import address_exists, interface_exists

from pyroute2 import NDB, netns

pytestmark = [require_root()]

test_matrix = make_test_matrix(dbs=['sqlite3/:memory:', 'postgres/pr2test'])


@pytest.mark.parametrize('context', test_matrix, indirect=True)
def test_create_remove(context):
    nsname = context.new_nsname
    with NDB(log=(context.new_log, logging.DEBUG)) as ndb:
        # create a netns via ndb.netns
        ndb.netns.create(nsname).commit()
        assert nsname in netns.listnetns()
        # remove the netns
        ndb.netns[nsname].remove().commit()
        assert nsname not in netns.listnetns()


@pytest.mark.parametrize('context', test_matrix, indirect=True)
def test_views_contain(context):
    nsname = context.new_nsname
    v0 = context.new_ifname
    v1 = context.new_ifname

    context.ndb.sources.add(netns=nsname)
    context.ndb.interfaces.create(
        **{
            'ifname': v0,
            'kind': 'veth',
            'peer': {'ifname': v1, 'net_ns_fd': nsname},
        }
    ).commit()

    assert v0 in context.ndb.interfaces
    assert v1 in context.ndb.interfaces  # should be fixed?
    assert {'ifname': v0, 'target': 'localhost'} in context.ndb.interfaces
    assert {'ifname': v1, 'target': nsname} in context.ndb.interfaces


@pytest.mark.parametrize('context', test_matrix, indirect=True)
def test_interface_move(context):
    ifname = context.new_ifname
    ifaddr = context.new_ipaddr
    nsname = context.new_nsname

    context.ndb.sources.add(netns=nsname)

    # create the interface
    (context.ndb.interfaces.create(ifname=ifname, kind='dummy').commit())

    # move it to a netns
    (context.ndb.interfaces[ifname].set('net_ns_fd', nsname).commit())

    # setup the interface only when it is moved
    (
        context.ndb.interfaces.wait(target=nsname, ifname=ifname)
        .set('state', 'up')
        .set('address', '00:11:22:33:44:55')
        .add_ip('%s/24' % ifaddr)
        .commit()
    )

    assert interface_exists(
        nsname, ifname=ifname, state='up', address='00:11:22:33:44:55'
    )


@pytest.mark.parametrize('context', test_matrix, indirect=True)
def test_source_basic(context):
    ifname = context.new_ifname
    ifaddr1 = context.new_ipaddr
    ifaddr2 = context.new_ipaddr
    ifaddr3 = context.new_ipaddr
    nsname = context.new_nsname

    context.ndb.sources.add(netns=nsname)

    (
        context.ndb.interfaces.create(
            target=nsname, ifname=ifname, kind='dummy'
        )
        .ipaddr.create(address=ifaddr1, prefixlen=24)
        .create(address=ifaddr2, prefixlen=24)
        .create(address=ifaddr3, prefixlen=24)
        .commit()
    )

    with NDB(
        sources=[{'target': 'localhost', 'netns': nsname, 'kind': 'netns'}]
    ) as ndb:
        if_idx = ndb.interfaces[ifname]['index']
        addr1_idx = ndb.addresses['%s/24' % ifaddr1]['index']
        addr2_idx = ndb.addresses['%s/24' % ifaddr2]['index']
        addr3_idx = ndb.addresses['%s/24' % ifaddr3]['index']

    assert if_idx == addr1_idx == addr2_idx == addr3_idx


@pytest.mark.parametrize('context', test_matrix, indirect=True)
def test_localhost_implicit(context):
    ifname = context.new_ifname
    ipaddr = context.new_ipaddr
    nsname = context.new_nsname

    context.ndb.sources.add(netns=nsname)
    context.ndb.localhost = nsname

    (
        context.ndb.interfaces.create(ifname=ifname, kind='dummy')
        .add_ip(address=ipaddr, prefixlen=24)
        .commit()
    )

    assert interface_exists(nsname, ifname=ifname)
    assert address_exists(nsname, ifname=ifname, address=ipaddr)


@pytest.mark.parametrize('context', test_matrix, indirect=True)
def test_localhost_explicit(context):
    ifname = context.new_ifname
    ipaddr = context.new_ipaddr
    nsname = context.new_nsname
    target = str(uuid.uuid4())

    context.ndb.sources.add(netns=nsname, target=target)
    context.ndb.localhost = target

    (
        context.ndb.interfaces.create(ifname=ifname, kind='dummy')
        .add_ip(address=ipaddr, prefixlen=24)
        .commit()
    )

    assert interface_exists(nsname, ifname=ifname)
    assert address_exists(nsname, ifname=ifname, address=ipaddr)



import pyroute2

source_suffix = '.rst'
master_doc = 'index'

project = 'lab.pyroute2'
copyright = '2022, Peter Saveliev'
author = 'Peter Saveliev'

release = pyroute2.__version__


extensions = [
]

templates_path = ['_templates']
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']


html_theme = 'default'
html_css_files = ['custom.css']
html_js_files = ['conf.js', 'lab.js', 'fixup.js']
html_static_path = ['_static']



import errno
import os
import platform
import pwd
import re
import stat
import subprocess
import sys
import uuid
from socket import AF_INET, AF_INET6

import netaddr
import pytest

from pyroute2 import config
from pyroute2.iproute.linux import IPRoute

try:
    import httplib
except ImportError:
    import http.client as httplib

dtcd_uuid = str(uuid.uuid4())
# check the dtcd
try:
    cx = httplib.HTTPConnection('localhost:7623')
    cx.request('GET', '/v1/network/')
    cx.getresponse()
    has_dtcd = True
except:
    has_dtcd = False

supernet = {
    AF_INET: netaddr.IPNetwork('172.16.0.0/12'),
    AF_INET6: netaddr.IPNetwork('fdb3:84e5:4ff4::/48'),
}
network_pool = {
    AF_INET: list(supernet[AF_INET].subnet(24)),
    AF_INET6: list(supernet[AF_INET6].subnet(64)),
}
allocations = {}
family_url = {AF_INET: 'ipv4', AF_INET6: 'ipv6'}


def allocate_network(family=AF_INET):
    global dtcd_uuid
    global network_pool
    global allocations

    network = None

    try:
        cx = httplib.HTTPConnection('localhost:7623')
        cx.request(
            'POST', '/v1/network/%s/' % family_url[family], body=dtcd_uuid
        )
        resp = cx.getresponse()
        if resp.status == 200:
            network = netaddr.IPNetwork(resp.read().decode('utf-8'))
        cx.close()
    except Exception:
        pass

    if network is None:
        network = network_pool[family].pop()
        allocations[network] = True

    return network


def free_network(network, family=AF_INET):
    global network_pool
    global allocations

    if network in allocations:
        allocations.pop(network)
        network_pool[family].append(network)
    else:
        cx = httplib.HTTPConnection('localhost:7623')
        cx.request(
            'DELETE', '/v1/network/%s/' % family_url[family], body=str(network)
        )
        cx.getresponse()
        cx.close()


def conflict_arch(arch):
    if platform.machine().find(arch) >= 0:
        pytest.skip('conflict with architecture %s' % (arch))


def kernel_version_ge(major, minor):
    # True if running kernel is >= X.Y
    if config.kernel[0] > major:
        return True
    if config.kernel[0] < major:
        return False
    if minor and config.kernel[1] < minor:
        return False
    return True


def require_kernel(major, minor=None):
    if not kernel_version_ge(major, minor):
        pytest.skip('incompatible kernel version')


def require_python(target):
    if sys.version_info[0] != target:
        pytest.skip('test requires Python %i' % target)


def require_8021q():
    try:
        os.stat('/proc/net/vlan/config')
    except OSError as e:
        # errno 2 'No such file or directory'
        if e.errno == 2:
            pytest.skip('missing 8021q support, or module is not loaded')
        raise


def require_bridge():
    with IPRoute() as ip:
        try:
            ip.link('add', ifname='test_req', kind='bridge')
        except Exception:
            pytest.skip('can not create <bridge>')
        idx = ip.link_lookup(ifname='test_req')
        if not idx:
            pytest.skip('can not create <bridge>')
        ip.link('del', index=idx)


def require_bond():
    with IPRoute() as ip:
        try:
            ip.link('add', ifname='test_req', kind='bond')
        except Exception:
            pytest.skip('can not create <bond>')
        idx = ip.link_lookup(ifname='test_req')
        if not idx:
            pytest.skip('can not create <bond>')
        ip.link('del', index=idx)


def require_user(user):
    if bool(os.environ.get('PYROUTE2_TESTS_RO', False)):
        pytest.skip('read-only tests requested')
    if pwd.getpwuid(os.getuid()).pw_name != user:
        pytest.skip('required user %s' % (user))


def require_executable(name):
    try:
        with open(os.devnull, 'w') as fnull:
            subprocess.check_call(['which', name], stdout=fnull, stderr=fnull)
    except Exception:
        pytest.skip('required %s not found' % (name))


def remove_link(name):
    if os.getuid() != 0:
        return
    with open(os.devnull, 'w') as fnull:
        subprocess.call(
            ['ip', 'link', 'del', 'dev', name], stdout=fnull, stderr=fnull
        )
    while True:
        links = get_ip_link()
        if name not in links:
            break


def create_link(name, kind):
    if os.getuid() != 0:
        return
    subprocess.call(['ip', 'link', 'add', 'dev', name, 'type', kind])

    for i in range(20):
        links = get_ip_link()
        if name in links:
            return

    raise Exception("interface not created")


def _check_output(*argv):
    # we can not use check_output, as it does not exist in 2.6
    process = subprocess.Popen(argv, stdout=subprocess.PIPE)
    ret = process.communicate()
    return ret[0].decode('utf-8').split('\n')


def grep(command, pattern=None):
    out = _check_output(*command.split())
    ret = []
    reg = re.compile(pattern)
    for string in out:
        if reg.search(string):
            ret.append(string)
    return ret


def get_ip_addr(interface=None):
    argv = ['ip', '-o', 'ad']
    if interface:
        argv.extend(['li', 'dev', interface])
    out = _check_output(*argv)
    ret = []
    for string in out:
        fields = string.split()
        if len(fields) >= 5 and fields[2][:4] == 'inet':
            ret.append(fields[3])
    return ret


def get_ip_brd(interface=None):
    argv = ['ip', '-o', 'ad']
    if interface:
        argv.extend(['li', 'dev', interface])
    out = _check_output(*argv)
    ret = []
    for string in out:
        fields = string.split()
        if len(fields) >= 5 and fields[4] == 'brd':
            ret.append(fields[5])
    return ret


def get_ip_link():
    ret = []
    out = _check_output('ip', '-o', 'li')
    for string in out:
        fields = string.split()
        if len(fields) >= 2:
            ret.append(fields[1][:-1].split('@')[0])
    return ret


def get_ip_default_routes():
    ret = []
    out = _check_output('ip', '-4', 'ro')
    for string in out:
        if 'default' in string:
            ret.append(string)
    return ret


def get_ip_rules(proto='-4'):
    ret = []
    out = _check_output('ip', proto, 'rule', 'show')
    for string in out:
        if len(string):
            ret.append(string)
    return ret


def count_socket_fds():
    pid_fd = '/proc/%s/fd' % os.getpid()
    sockets = 0
    for fd in os.listdir(pid_fd):
        try:
            if stat.S_ISSOCK(os.stat(os.path.join(pid_fd, fd)).st_mode):
                sockets += 1
        except OSError as e:
            if e.errno != errno.ENOENT:
                raise
    return sockets



import collections
import json
import subprocess

import pytest
from pr2test.marks import require_root

from pyroute2 import NDB
from pyroute2.common import dqn2int, hexdump, hexload
from pyroute2.dhcp import client

pytestmark = [require_root()]


@pytest.fixture
def ctx():
    ndb = NDB()
    index = 0
    ifname = ''
    # get a DHCP default route, if exists
    for route in ndb.routes.dump().filter(proto=16, dst=''):
        index = route.oif
        ifname = ndb.interfaces[index]['ifname']
    yield collections.namedtuple('Context', ['ndb', 'index', 'ifname'])(
        ndb, index, ifname
    )
    ndb.close()


def test_client_module(ctx):
    if ctx.index == 0:
        pytest.skip('no DHCP interfaces detected')

    response = client.action(ctx.ifname)
    options = response['options']
    router = response['options']['router'][0]
    prefixlen = dqn2int(response['options']['subnet_mask'])
    address = response['yiaddr']
    l2addr = response['chaddr']

    # convert addresses like 96:0:1:45:fa:6c into 96:00:01:45:fa:6c
    assert (
        hexdump(hexload(l2addr)) == ctx.ndb.interfaces[ctx.ifname]['address']
    )
    assert router == ctx.ndb.routes['default']['gateway']
    assert {
        'address': address,
        'prefixlen': prefixlen,
        'index': ctx.index,
    } in ctx.ndb.addresses
    assert options['lease_time'] > 0
    return response


def test_client_console(ctx):
    response_from_module = json.loads(json.dumps(test_client_module(ctx)))
    client = subprocess.run(
        ['pyroute2-dhcp-client', ctx.ifname], stdout=subprocess.PIPE
    )
    response_from_console = json.loads(client.stdout)
    assert response_from_module == response_from_console



import pytest
from pr2test.context_manager import make_test_matrix, skip_if_not_implemented
from pr2test.marks import require_root
from pr2test.tools import neighbour_exists

pytestmark = [require_root()]

test_matrix = make_test_matrix(
    targets=['local', 'netns'], dbs=['sqlite3/:memory:', 'postgres/pr2test']
)


@pytest.mark.parametrize('context', test_matrix, indirect=True)
def test_add_neighbour_simple(context):
    ifname = context.new_ifname
    ipaddr = context.new_ipaddr
    neighbour = context.new_ipaddr

    (
        context.ndb.interfaces.create(ifname=ifname, kind='dummy', state='up')
        .add_ip(address=ipaddr, prefixlen=24)
        .commit()
    )
    (
        context.ndb.neighbours.create(
            ifindex=context.ndb.interfaces[ifname]['index'],
            dst=neighbour,
            lladdr='00:11:22:33:44:55',
        ).commit()
    )

    assert neighbour_exists(
        context.netns,
        ifindex=context.ndb.interfaces[ifname]['index'],
        dst=neighbour,
        lladdr='00:11:22:33:44:55',
    )


@pytest.mark.parametrize('context', test_matrix, indirect=True)
def test_add_neighbour_chain(context):
    ifname = context.new_ifname
    ipaddr = context.new_ipaddr
    neighbour = context.new_ipaddr

    (
        context.ndb.interfaces.create(ifname=ifname, kind='dummy', state='up')
        .ipaddr.create(address=ipaddr, prefixlen=24)
        .commit()
        .chain.neighbours.create(dst=neighbour, lladdr='00:11:22:33:44:55')
        .commit()
    )

    assert neighbour_exists(
        context.netns,
        ifindex=context.ndb.interfaces[ifname]['index'],
        dst=neighbour,
        lladdr='00:11:22:33:44:55',
    )


@pytest.mark.parametrize('context', test_matrix, indirect=True)
@skip_if_not_implemented
def test_add_neighbour_method(context):
    ifname = context.new_ifname
    ipaddr = context.new_ipaddr
    neighbour = context.new_ipaddr

    (
        context.ndb.interfaces.create(ifname=ifname, kind='dummy', state='up')
        .add_ip(address=ipaddr, prefixlen=24)
        .add_neighbour(dst=neighbour, lladdr='00:11:22:33:44:55')
        .commit()
    )

    assert neighbour_exists(
        context.netns,
        ifindex=context.ndb.interfaces[ifname]['index'],
        dst=neighbour,
        lladdr='00:11:22:33:44:55',
    )



from pr2test.context_manager import skip_if_not_supported

from pyroute2 import DL


@skip_if_not_supported
def test_list():
    with DL() as dl:
        dls = dl.get_dump()
        if not dls:
            raise RuntimeError('no devlink devices found')

        assert dl.list()



import pyroute2

extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.inheritance_diagram',
    'aafigure.sphinxext',
]

aafig_format = {'html': 'svg', 'man': None}

inheritance_graph_attrs = {'rankdir': 'LR', 'ratio': 'auto'}
source_suffix = '.rst'
master_doc = 'index'
project = u'pyroute2'
copyright = u'Peter Saveliev and PyRoute2 team'

release = pyroute2.__version__

exclude_patterns = ['_build']
pygments_style = 'sphinx'
autodoc_member_order = 'bysource'

html_theme = 'default'
html_static_path = ['_static']
html_js_files = ['fixup.js']
html_css_files = ['custom.css']
htmlhelp_basename = 'pyroute2doc'
templates_path = ['_templates']


man_pages = [
    (
        'pyroute2-cli',
        'pyroute2-cli',
        'pyroute2 command line interface',
        ['Peter Saveliev'],
        1,
    ),
    (
        'pyroute2-dhcp-client',
        'pyroute2-dhcp-client',
        'pyroute2 dhcp client',
        ['Peter Saveliev'],
        1,
    ),
]



from unittest import mock


class TokenManager(mock.Mock):
    def __init__(self, *argv, **kwarg):
        super().__init__(*argv, **kwarg)
        self.validate = mock.Mock(name='validate')
        self.validate.return_value = {'expires_at': '3022.07.04 00:00 CEST'}



import pytest
from pr2test.marks import require_root
from pr2test.tools import address_exists

from pyroute2 import NDB

pytestmark = [require_root()]


@pytest.mark.xfail(reason='flaky test, only to collect failure logs')
def test_add_del_ip_dict(context):
    ifname = context.new_ifname
    ifaddr1 = context.new_ipaddr
    ifaddr2 = context.new_ipaddr
    log_spec = (
        context.spec.log_spec[0] + '.chaotic',
        context.spec.log_spec[1],
    )

    with NDB(
        log=log_spec,
        sources=[
            {
                'target': 'localhost',
                'kind': 'ChaoticIPRoute',
                'success_rate': 0.98,
            }
        ],
    ) as test_ndb:

        (
            test_ndb.interfaces.create(
                ifname=ifname, kind='dummy', state='down'
            )
            .add_ip({'address': ifaddr1, 'prefixlen': 24})
            .add_ip({'address': ifaddr2, 'prefixlen': 24})
            .commit()
        )

        assert address_exists(context.netns, ifname=ifname, address=ifaddr1)
        assert address_exists(context.netns, ifname=ifname, address=ifaddr2)

        (
            test_ndb.interfaces[{'ifname': ifname}]
            .del_ip({'address': ifaddr2, 'prefixlen': 24})
            .del_ip({'address': ifaddr1, 'prefixlen': 24})
            .commit()
        )

        assert not address_exists(
            context.netns, ifname=ifname, address=ifaddr1
        )
        assert not address_exists(
            context.netns, ifname=ifname, address=ifaddr2
        )



from inspect import signature

import pytest

import pyroute2
from pyroute2 import netlink, netns
from pyroute2.netlink import exceptions, rtnl
from pyroute2.netlink.rtnl import ifinfmsg, ndmsg


def parameters(func):
    try:
        return set(signature(func).parameters.keys())
    except ValueError:
        pytest.skip('ginature check error, skip test')


def test_imports():
    assert parameters(pyroute2.NetNS) > set(('netns', 'flags', 'libc'))
    assert signature(pyroute2.IPRoute)
    assert issubclass(netlink.NetlinkError, Exception)
    assert issubclass(exceptions.NetlinkDumpInterrupted, Exception)
    assert netlink.NetlinkError == exceptions.NetlinkError
    assert netlink.nla_slot
    assert netlink.nla_base
    assert parameters(rtnl.rt_scope.get) == set(('key', 'default'))
    assert isinstance(rtnl.rt_proto, dict) and 'static' in rtnl.rt_proto
    assert parameters(netns._create) == set(('netns', 'libc', 'pid'))
    assert parameters(netns.remove) == set(('netns', 'libc'))
    assert parameters(netns.listnetns) == set(('nspath',))
    assert ifinfmsg.IFF_ALLMULTI == 0x200
    assert {state[1]: state[0] for state in ndmsg.states.items()} == {
        0: 'none',
        1: 'incomplete',
        2: 'reachable',
        4: 'stale',
        8: 'delay',
        16: 'probe',
        32: 'failed',
        64: 'noarp',
        128: 'permanent',
    }



import getpass

import pytest

from pyroute2 import IPRoute
from pyroute2.common import uifname
from pyroute2.netlink import nlmsg

pytestmark = [
    pytest.mark.skipif(getpass.getuser() != 'root', reason='no root access')
]


@pytest.fixture
def ipr():
    iproute = IPRoute()
    iproute.default_ifname = uifname()
    yield iproute
    index = iproute.link_lookup(ifname=iproute.default_ifname)
    if index:
        iproute.link('del', index=index)
    iproute.close()


def test_dump(ipr):
    assert all([isinstance(message, nlmsg) for message in ipr.dump()])


def test_tuntap(ipr):
    ipr.link('add', ifname=ipr.default_ifname, kind='tuntap', mode='tun')
    ipr.poll(
        ipr.link, 'dump', timeout=5, ifname=ipr.default_ifname, kind='tun'
    )


def test_bridge(ipr):
    ipr.link('add', ifname=ipr.default_ifname, kind='bridge')
    interface = ipr.poll(
        ipr.link,
        'dump',
        timeout=5,
        ifname=ipr.default_ifname,
        kind='bridge',
        br_stp_state=0,
    )[0]
    ipr.link('set', index=interface['index'], kind='bridge', br_stp_state=1)
    ipr.poll(
        ipr.link,
        'dump',
        timeout=5,
        index=interface['index'],
        kind='bridge',
        br_stp_state=1,
    )



import io
import re

import pytest
from setuptools._vendor import packaging


@pytest.fixture
def files():
    context = {}
    for file in ('VERSION', 'CHANGELOG.rst'):
        with open(file, 'r') as f:
            obj = io.StringIO()
            obj.write(f.read())
            obj.seek(0)
            context[file] = obj
    yield context


def test_static_version_file(files):
    assert re.match(
        r'^[0-9]\.[0-9]\.[0-9]{1,2}(\.post[0-9]+|\.rc[0-9]+){0,1}$',
        files['VERSION'].getvalue().strip(),
    )


def test_changelog(files):
    line = ''
    for line in files['CHANGELOG.rst'].readlines():
        if line[0] == '*':
            break
    static_version = packaging.version.parse(files['VERSION'].getvalue())
    last_changelog_version = packaging.version.parse(line.split()[1])
    assert static_version >= last_changelog_version



from setup import lab
from task import IPRoute

ipr = lab.registry[0]
if not isinstance(ipr, IPRoute):
    raise AssertionError('expected IPRoute instance')

ipr.get_links.assert_called()
ipr.close.assert_called_once()



import errno
import queue
import struct

from pyroute2.lab import LAB_API
from pyroute2.netlink.nlsocket import Stats
from pyroute2.netlink.rtnl.ifaddrmsg import ifaddrmsg
from pyroute2.netlink.rtnl.ifinfmsg import ifinfmsg

DUMP_LINKS = [
    {
        'attrs': [
            ['IFLA_IFNAME', 'lo'],
            ['IFLA_TXQLEN', 1000],
            ['IFLA_OPERSTATE', 'UNKNOWN'],
            ['IFLA_LINKMODE', 0],
            ['IFLA_MTU', 65536],
            ['IFLA_MIN_MTU', 0],
            ['IFLA_MAX_MTU', 0],
            ['IFLA_GROUP', 0],
            ['IFLA_PROMISCUITY', 0],
            ['IFLA_NUM_TX_QUEUES', 1],
            ['IFLA_GSO_MAX_SEGS', 65535],
            ['IFLA_GSO_MAX_SIZE', 65536],
            ['IFLA_GRO_MAX_SIZE', 65536],
            ['IFLA_NUM_RX_QUEUES', 1],
            ['IFLA_CARRIER', 1],
            ['IFLA_QDISC', 'noqueue'],
            ['IFLA_CARRIER_CHANGES', 0],
            ['IFLA_CARRIER_UP_COUNT', 0],
            ['IFLA_CARRIER_DOWN_COUNT', 0],
            ['IFLA_PROTO_DOWN', 0],
            [
                'IFLA_MAP',
                {
                    'base_addr': 0,
                    'dma': 0,
                    'irq': 0,
                    'mem_end': 0,
                    'mem_start': 0,
                    'port': 0,
                },
            ],
            ['IFLA_ADDRESS', '00:00:00:00:00:00'],
            ['IFLA_BROADCAST', '00:00:00:00:00:00'],
            [
                'IFLA_STATS64',
                {
                    'collisions': 0,
                    'multicast': 0,
                    'rx_bytes': 43309665,
                    'rx_compressed': 0,
                    'rx_crc_errors': 0,
                    'rx_dropped': 0,
                    'rx_errors': 0,
                    'rx_fifo_errors': 0,
                    'rx_frame_errors': 0,
                    'rx_length_errors': 0,
                    'rx_missed_errors': 0,
                    'rx_over_errors': 0,
                    'rx_packets': 173776,
                    'tx_aborted_errors': 0,
                    'tx_bytes': 43309665,
                    'tx_carrier_errors': 0,
                    'tx_compressed': 0,
                    'tx_dropped': 0,
                    'tx_errors': 0,
                    'tx_fifo_errors': 0,
                    'tx_heartbeat_errors': 0,
                    'tx_packets': 173776,
                    'tx_window_errors': 0,
                },
            ],
            [
                'IFLA_STATS',
                {
                    'collisions': 0,
                    'multicast': 0,
                    'rx_bytes': 43309665,
                    'rx_compressed': 0,
                    'rx_crc_errors': 0,
                    'rx_dropped': 0,
                    'rx_errors': 0,
                    'rx_fifo_errors': 0,
                    'rx_frame_errors': 0,
                    'rx_length_errors': 0,
                    'rx_missed_errors': 0,
                    'rx_over_errors': 0,
                    'rx_packets': 173776,
                    'tx_aborted_errors': 0,
                    'tx_bytes': 43309665,
                    'tx_carrier_errors': 0,
                    'tx_compressed': 0,
                    'tx_dropped': 0,
                    'tx_errors': 0,
                    'tx_fifo_errors': 0,
                    'tx_heartbeat_errors': 0,
                    'tx_packets': 173776,
                    'tx_window_errors': 0,
                },
            ],
            ['IFLA_XDP', {'attrs': [['IFLA_XDP_ATTACHED', None]]}],
            [
                'IFLA_AF_SPEC',
                {
                    'attrs': [
                        [
                            'AF_INET',
                            {
                                'accept_local': 0,
                                'accept_redirects': 1,
                                'accept_source_route': 0,
                                'arp_accept': 0,
                                'arp_announce': 0,
                                'arp_ignore': 0,
                                'arp_notify': 0,
                                'arpfilter': 0,
                                'bootp_relay': 0,
                                'dummy': 65672,
                                'force_igmp_version': 0,
                                'forwarding': 1,
                                'igmpv2_unsolicited_report_interval': 10000,
                                'igmpv3_unsolicited_report_interval': 1000,
                                'log_martians': 0,
                                'mc_forwarding': 0,
                                'medium_id': 0,
                                'nopolicy': 1,
                                'noxfrm': 1,
                                'promote_secondaries': 1,
                                'proxy_arp': 0,
                                'proxy_arp_pvlan': 0,
                                'route_localnet': 0,
                                'rp_filter': 2,
                                'secure_redirects': 1,
                                'send_redirects': 1,
                                'shared_media': 1,
                                'src_vmark': 0,
                                'tag': 0,
                            },
                        ]
                    ]
                },
            ],
        ],
        'change': 0,
        'event': 'RTM_NEWLINK',
        'family': 0,
        'flags': 65609,
        'header': {
            'error': None,
            'flags': 2,
            'length': 1364,
            'pid': 303471,
            'sequence_number': 260,
            'stats': Stats(qsize=0, delta=0, delay=0),
            'target': 'localhost',
            'type': 16,
        },
        'ifi_type': 772,
        'index': 1,
        'state': 'up',
    }
]

DUMP_ADDRS = [
    {
        'attrs': [
            ['IFA_ADDRESS', '127.0.0.1'],
            ['IFA_LOCAL', '127.0.0.1'],
            ['IFA_LABEL', 'lo'],
            ['IFA_FLAGS', 128],
            [
                'IFA_CACHEINFO',
                {
                    'cstamp': 155,
                    'ifa_preferred': 4294967295,
                    'ifa_valid': 4294967295,
                    'tstamp': 155,
                },
            ],
        ],
        'event': 'RTM_NEWADDR',
        'family': 2,
        'flags': 128,
        'header': {
            'error': None,
            'flags': 2,
            'length': 76,
            'pid': 303471,
            'sequence_number': 263,
            'stats': Stats(qsize=0, delta=0, delay=0),
            'target': 'localhost',
            'type': 20,
        },
        'index': 1,
        'prefixlen': 8,
        'scope': 254,
    }
]


class IPRoute(LAB_API):
    def __init__(self, *argv, **kwarg):
        super().__init__(*argv, **kwarg)
        self.output_queue = queue.Queue(maxsize=512)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()

    def close(self, code=errno.ECONNRESET):
        self.output_queue.put(struct.pack('IHHQIQQ', 28, 2, 0, 0, code, 0, 0))

    def dump(self):
        for method in (self.get_links, self.get_addr):
            for msg in method():
                yield msg

    def _get_dump(self, dump, msg_class):
        for data in dump:
            loader = msg_class()
            loader.load(data)
            loader.encode()
            msg = msg_class()
            msg.data = loader.data
            msg.decode()
            yield msg

    def get_addr(self):
        return self._get_dump(DUMP_ADDRS, ifaddrmsg)

    def get_links(self):
        return self._get_dump(DUMP_LINKS, ifinfmsg)


class ChaoticIPRoute:
    def __init__(self, *argv, **kwarg):
        raise NotImplementedError()


class RawIPRoute:
    def __init__(self, *argv, **kwarg):
        raise NotImplementedError()



import ast
import collections
import inspect
import os
import sys

import nox
import pytest

# load nox sessions into the registry
import noxfile  # noqa: F401

# load all defined sessions using nox discovery
nox_sessions = nox.registry.get().values()


@pytest.fixture
def session(request):
    # get closure vars, if any
    cvars = inspect.getclosurevars(request.param.func).nonlocals
    yield collections.namedtuple('Session', ('src_func', 'has_user_config'))(
        cvars['func'] if cvars and 'func' in cvars else request.param.func,
        hasattr(request.param.func, '__has_user_config__'),
    )


@pytest.mark.skipif(
    sys.version_info < (3, 8), reason='unsupported Python version'
)
@pytest.mark.parametrize('session', nox_sessions, indirect=True)
def test_options_call(session):
    # walk the AST tree
    for node in ast.walk(ast.parse(inspect.getsource(session.src_func))):
        # filter only plain function calls, no attributes
        if isinstance(node, ast.Call) and isinstance(node.func, ast.Name):
            # lookup call `options(...)`
            if node.func.id == 'options':
                # only for decorated by `add_session_config`
                assert session.has_user_config
                # check the arguments
                assert len(node.args) == 2
                assert isinstance(node.args[0], ast.Constant)
                assert isinstance(node.args[1], ast.Name)
                assert node.args[1].id == 'config'


@pytest.mark.parametrize('session', nox_sessions, indirect=True)
def test_session_parameters(session):
    args = inspect.getfullargspec(session.src_func).args
    if session.has_user_config:
        assert args == ['session', 'config']
    else:
        assert args == ['session']


@pytest.mark.skipif(
    sys.version_info < (3, 8), reason='unsupported Python version'
)
@pytest.mark.parametrize('session', nox_sessions, indirect=True)
def test_requirements_files(session):
    for node in ast.walk(ast.parse(inspect.getsource(session.src_func))):
        #
        # inspect function calls, filter direct or indirect install
        if isinstance(node, ast.Call):
            if (
                isinstance(node.func, ast.Name)
                and node.func.id == 'setup_venv_common'
            ):
                #
                # inspect calls `setup_venv_common(...)` -- indirect install
                assert len(node.args) in (1, 2)
                if len(node.args) == 2:
                    assert isinstance(node.args[1], ast.Constant)
                    flavour = node.args[1].value
                else:
                    flavour = (
                        inspect.signature(noxfile.setup_venv_common)
                        .parameters['flavour']
                        .default
                    )
                assert os.stat(f'requirements.{flavour}.txt')
            elif (
                isinstance(node.func, ast.Attribute)
                and isinstance(node.func.value, ast.Name)
                and node.func.attr == 'install'
                and node.args[0].value == '-r'
            ):
                #
                # inspect call `session.install('-r', ...)` -- direct install
                assert os.stat(node.args[1].value)



import ctypes
import ctypes.util
import re
import subprocess

import pytest
from pr2test.context_manager import skip_if_not_supported
from pr2test.marks import require_root

from pyroute2 import protocols
from pyroute2.netlink.rtnl import TC_H_INGRESS

pytestmark = [require_root()]


def get_bpf_syscall_num():
    # determine bpf syscall number
    prog = """
#include <asm/unistd.h>
#define XSTR(x) STR(x)
#define STR(x) #x
#pragma message "__NR_bpf=" XSTR(__NR_bpf)
"""
    cmd = ['gcc', '-x', 'c', '-c', '-', '-o', '/dev/null']
    gcc = subprocess.Popen(cmd, stdin=subprocess.PIPE, stderr=subprocess.PIPE)
    out = gcc.communicate(input=prog.encode('ascii'))[1]
    m = re.search('__NR_bpf=([0-9]+)', str(out))
    if not m:
        pytest.skip('bpf syscall not available')
    return int(m.group(1))


def get_simple_bpf_program(prog_type):
    NR_bpf = get_bpf_syscall_num()

    class BPFAttr(ctypes.Structure):
        _fields_ = [
            ('prog_type', ctypes.c_uint),
            ('insn_cnt', ctypes.c_uint),
            ('insns', ctypes.POINTER(ctypes.c_ulonglong)),
            ('license', ctypes.c_char_p),
            ('log_level', ctypes.c_uint),
            ('log_size', ctypes.c_uint),
            ('log_buf', ctypes.c_char_p),
            ('kern_version', ctypes.c_uint),
        ]

    BPF_PROG_TYPE_SCHED_CLS = 3
    BPF_PROG_TYPE_SCHED_ACT = 4
    BPF_PROG_LOAD = 5
    insns = (ctypes.c_ulonglong * 2)()
    # equivalent to: int my_func(void *) { return 1; }
    insns[0] = 0x00000001000000B7
    insns[1] = 0x0000000000000095
    license = ctypes.c_char_p(b'GPL')
    if prog_type.lower() == "sched_cls":
        attr = BPFAttr(
            BPF_PROG_TYPE_SCHED_CLS, len(insns), insns, license, 0, 0, None, 0
        )
    elif prog_type.lower() == "sched_act":
        attr = BPFAttr(
            BPF_PROG_TYPE_SCHED_ACT, len(insns), insns, license, 0, 0, None, 0
        )
    libc = ctypes.CDLL(ctypes.util.find_library('c'))
    libc.syscall.argtypes = [
        ctypes.c_long,
        ctypes.c_int,
        ctypes.POINTER(type(attr)),
        ctypes.c_uint,
    ]
    libc.syscall.restype = ctypes.c_int
    fd = libc.syscall(NR_bpf, BPF_PROG_LOAD, attr, ctypes.sizeof(attr))
    return fd


@pytest.fixture
def bpf_cls():
    fd = get_simple_bpf_program('sched_cls')
    if fd == -1:
        pytest.skip('bpf syscall error')
    yield fd


@pytest.fixture
def bpf_act():
    fd = get_simple_bpf_program('sched_act')
    if fd == -1:
        pytest.skip('bpf syscall error')
    yield fd


@pytest.fixture
def ingress(context):
    context.ipr.tc(
        'add',
        kind='ingress',
        index=context.default_interface.index,
        handle=0xFFFF0000,
    )
    yield context


@skip_if_not_supported
def test_simple(ingress):
    qds = [
        x
        for x in ingress.ipr.get_qdiscs()
        if x['index'] == ingress.default_interface.index
    ]
    # assert the list is not empty
    assert qds
    # assert there is the ingress queue
    for qd in qds:
        if qd.get_attr('TCA_KIND') == 'ingress':
            # assert it has proper handle and parent
            assert qd['handle'] == 0xFFFF0000
            assert qd['parent'] == TC_H_INGRESS
            break
    else:
        raise Exception('no ingress qdisc found')


@skip_if_not_supported
def test_filter_sched_cls(ingress, bpf_cls):
    ingress.ipr.tc(
        'add-filter',
        kind='bpf',
        index=ingress.default_interface.index,
        handle=0,
        fd=bpf_cls,
        name='my_func',
        parent=0xFFFF0000,
        action='ok',
        classid=1,
        rate='10kbit',
        burst=10240,
        mtu=2040,
    )
    fls = ingress.ipr.get_filters(
        index=ingress.default_interface.index, parent=0xFFFF0000
    )
    # assert the supplied policer is returned to us intact
    plcs = [
        x
        for x in fls
        if x.get_attr('TCA_OPTIONS') is not None
        and (x.get_attr('TCA_OPTIONS').get_attr('TCA_BPF_POLICE') is not None)
    ][0]
    options = plcs.get_attr('TCA_OPTIONS')
    police = options.get_attr('TCA_BPF_POLICE').get_attr('TCA_POLICE_TBF')
    assert police['rate'] == 1250
    assert police['mtu'] == 2040


@skip_if_not_supported
def test_filter_sched_act(ingress, bpf_cls, bpf_act):
    index, ifname = ingress.default_interface
    ingress.ipr.tc(
        'add-filter',
        'bpf',
        index=index,
        handle=0,
        fd=bpf_cls,
        name='my_func',
        parent=0xFFFF0000,
        action='ok',
        classid=1,
    )
    action = {'kind': 'bpf', 'fd': bpf_act, 'name': 'my_func', 'action': 'ok'}
    ingress.ipr.tc(
        'add-filter',
        'u32',
        index=index,
        handle=1,
        protocol=protocols.ETH_P_ALL,
        parent=0xFFFF0000,
        target=0x10002,
        keys=['0x0/0x0+0'],
        action=action,
    )
    fls = ingress.ipr.get_filters(index=index, parent=0xFFFF0000)
    assert fls
    bpf_filter = [
        x
        for x in fls
        if x.get_attr('TCA_OPTIONS') is not None
        and (x.get_attr('TCA_OPTIONS').get_attr('TCA_BPF_ACT') is not None)
    ][0]
    bpf_options = bpf_filter.get_attr('TCA_OPTIONS')
    assert bpf_options.get_attr('TCA_BPF_NAME') == 'my_func'
    gact_parms = (
        bpf_options.get_attr('TCA_BPF_ACT')
        .get_attr('TCA_ACT_PRIO_1')
        .get_attr('TCA_ACT_OPTIONS')
        .get_attr('TCA_GACT_PARMS')
    )
    assert gact_parms['action'] == 0


@skip_if_not_supported
def test_filter_delete(context, bpf_cls):
    context.ipr.tc('add', kind='clsact', index=context.default_interface.index)
    context.ipr.tc(
        'add-filter',
        kind='bpf',
        index=context.default_interface.index,
        fd=bpf_cls,
        name='my_func',
        parent='ffff:fff2',
        classid=1,
        direct_action=True,
    )
    filters = context.ipr.get_filters(
        index=context.default_interface.index, parent='ffff:fff2'
    )
    # len == 2: handles 0 and 1
    assert len(filters) == 2
    context.ipr.tc(
        'del-filter',
        kind='bpf',
        index=context.default_interface.index,
        parent='ffff:fff2',
        info=filters[0]['info'],
    )
    filters = context.ipr.get_filters(
        index=context.default_interface.index, parent='ffff:fff2'
    )
    assert len(filters) == 0



import collections
import errno

import pytest
from pr2test.marks import require_root

from pyroute2 import IW, IPRoute
from pyroute2.netlink.exceptions import NetlinkError


@pytest.fixture
def ctx():
    iw = None
    ifname = None
    wiphy = None
    index = None
    try:
        iw = IW()
    except NetlinkError as e:
        if e.code == errno.ENOENT:
            pytest.skip('nl80211 not supported')
        raise
    ifaces = iw.get_interfaces_dump()
    if not ifaces:
        raise pytest.skip('no wireless interfaces found')
    for i in ifaces:
        ifname = i.get_attr('NL80211_ATTR_IFNAME')
        index = i.get_attr('NL80211_ATTR_IFINDEX')
        wiphy = i.get_attr('NL80211_ATTR_WIPHY')
        if index:
            break
    else:
        pytest.skip('can not detect the interface to use')

    yield collections.namedtuple(
        'WirelessContext', ['iw', 'ifname', 'index', 'wiphy']
    )(iw, ifname, index, wiphy)

    iw.close()


def test_list_wiphy(ctx):
    ctx.iw.list_wiphy()


def test_list_dev(ctx):
    ctx.iw.list_dev()


@require_root
def test_scan(ctx):
    with IPRoute() as ipr:
        ipr.link('set', index=ctx.index, state='up')
    ctx.iw.scan(ctx.index)



import random

import pytest
from pr2test.context_manager import make_test_matrix
from pr2test.marks import require_root
from pr2test.tools import address_exists, interface_exists, route_exists

from pyroute2.netlink.rtnl.rtmsg import IP6_RT_PRIO_USER, rtmsg

pytestmark = [require_root()]


test_matrix_simple = make_test_matrix(
    targets=['local', 'netns'], dbs=['sqlite3/:memory:', 'postgres/pr2test']
)


@pytest.mark.parametrize('context', test_matrix_simple, indirect=True)
def test_table_undef(context):
    ipaddr1 = context.new_ip6addr
    ipaddr2 = context.new_ip6addr
    index, ifname = context.default_interface
    (
        context.ndb.routes.create(
            dst=ipaddr1, dst_len=128, table=5000, oif=index
        ).commit()
    )
    assert route_exists(context.netns, dst=ipaddr1, table=5000)
    assert not route_exists(context.netns, dst=ipaddr2, table=5000)
    assert context.ndb.routes[f'{ipaddr1}/128']['oif'] == index
    with pytest.raises(KeyError):
        context.ndb.routes[f'{ipaddr2}/128']


test_matrix_scopes = make_test_matrix(
    targets=['local', 'netns'],
    tables=[
        (None, 0),
        (None, 200),
        (None, 253),
        (6001, 0),
        (6001, 200),
        (6001, 253),
        (None, 'universe'),
        (None, 'site'),
        (None, 'link'),
        (6001, 'universe'),
        (6001, 'site'),
        (6001, 'link'),
    ],
    dbs=['sqlite3/:memory:', 'postgres/pr2test'],
)


@pytest.mark.parametrize('context', test_matrix_scopes, indirect=True)
def test_scopes(context):
    ipaddr = context.new_ipaddr
    ifname = context.new_ifname
    table, scope = context.table
    dst = '172.24.200.142'
    (
        context.ndb.interfaces.create(ifname=ifname, kind='dummy', state='up')
        .add_ip(address=ipaddr, prefixlen=24)
        .commit()
    )
    spec = {
        'dst': dst,
        'oif': context.ndb.interfaces[ifname]['index'],
        'dst_len': 32,
        'scope': scope,
    }
    if table:
        spec['table'] = table
    (context.ndb.routes.create(**spec).commit())
    assert interface_exists(context.netns, ifname=ifname)
    assert route_exists(context.netns, **spec)
    (context.ndb.routes[spec].remove().commit())
    assert not route_exists(context.netns, **spec)


test_matrix_flags = make_test_matrix(
    targets=['local', 'netns'],
    tables=[
        (None, 0),
        (None, 4),
        (None, 'onlink'),
        (None, ['onlink']),
        (6001, 0),
        (6001, 4),
        (6001, 'onlink'),
        (6001, ['onlink']),
    ],
    dbs=['sqlite3/:memory:', 'postgres/pr2test'],
)


@pytest.mark.parametrize('context', test_matrix_flags, indirect=True)
def test_flags(context):
    ipaddr = context.new_ipaddr
    ifname = context.new_ifname
    table, flags = context.table
    dst = '172.24.200.142'
    (
        context.ndb.interfaces.create(ifname=ifname, kind='dummy', state='up')
        .add_ip(address=ipaddr, prefixlen=24)
        .commit()
    )
    spec = {
        'dst': dst,
        'oif': context.ndb.interfaces[ifname]['index'],
        'dst_len': 32,
        'flags': flags,
        'gateway': context.new_ipaddr,
    }
    if table:
        spec['table'] = table
    (context.ndb.routes.create(**spec).commit())
    assert interface_exists(context.netns, ifname=ifname)
    assert route_exists(context.netns, **spec)
    (context.ndb.routes[spec].remove().commit())
    assert not route_exists(context.netns, **spec)


test_matrix = make_test_matrix(
    targets=['local', 'netns'],
    tables=[None, 501, 5001],
    dbs=['sqlite3/:memory:', 'postgres/pr2test'],
)


@pytest.mark.parametrize('context', test_matrix, indirect=True)
def test_ipv6_default_priority(context):
    ifname = context.new_ifname
    ipaddr = context.new_ip6addr
    table = context.table
    (
        context.ndb.interfaces.create(ifname=ifname, kind='dummy', state='up')
        .add_ip(f'{ipaddr}/64')
        .commit()
    )
    dst = 'beef:feed:fade::'
    parameters = {
        'dst': f'{dst}/112',
        'oif': context.ndb.interfaces[ifname]['index'],
        'priority': 0,
        'table': table,
    }
    context.ndb.routes.create(**parameters).commit()
    assert route_exists(context.netns, dst=dst, table=table or 254)
    assert context.ndb.routes[parameters]['priority'] == IP6_RT_PRIO_USER


@pytest.mark.parametrize('context', test_matrix, indirect=True)
def test_empty_target(context):
    ipaddr = context.new_ip6addr
    table = context.table
    index, ifname = context.default_interface
    (
        context.ndb.routes.create(
            dst=ipaddr, dst_len=128, oif=index, table=table
        ).commit()
    )
    assert route_exists(context.netns, dst=ipaddr, table=table or 254)
    (
        context.ndb.routes[{'table': table, 'dst': f'{ipaddr}/128'}]
        .remove()
        .commit()
    )
    assert not route_exists(context.netns, dst=ipaddr, table=table or 254)


@pytest.mark.parametrize('context', test_matrix, indirect=True)
def test_basic(context):

    ifaddr = context.new_ipaddr
    router = context.new_ipaddr
    ifname = context.new_ifname
    ipnet = str(context.ipnets[1].network)
    table = context.table

    (
        context.ndb.interfaces.create(ifname=ifname, kind='dummy', state='up')
        .ipaddr.create(address=ifaddr, prefixlen=24)
        .commit()
    )

    spec = {'dst_len': 24, 'dst': ipnet, 'gateway': router}

    if table:
        spec['table'] = table

    (context.ndb.routes.create(**spec).commit())

    assert interface_exists(context.netns, ifname=ifname)
    assert address_exists(context.netns, ifname=ifname, address=ifaddr)
    assert route_exists(context.netns, dst=ipnet, table=table or 254)


@pytest.mark.parametrize('context', test_matrix, indirect=True)
def test_default(context):

    ifaddr = context.new_ipaddr
    router = context.new_ipaddr
    ifname = context.new_ifname
    random.seed()
    tnum = random.randint(500, 600)

    (
        context.ndb.interfaces.create(ifname=ifname, kind='dummy', state='up')
        .add_ip('%s/24' % ifaddr)
        .commit()
    )

    spec = {'dst': 'default', 'gateway': router}

    if context.table:
        table = context.table
    else:
        table = tnum
    spec['table'] = table

    (context.ndb.routes.create(**spec).commit())

    assert address_exists(context.netns, ifname=ifname, address=ifaddr)
    assert route_exists(context.netns, gateway=router, table=table)


@pytest.mark.parametrize('context', test_matrix, indirect=True)
def test_spec(context):

    ipaddr = context.new_ipaddr
    router = context.new_ipaddr
    ifname = context.new_ifname
    net = str(context.ipnets[1].network)
    table = context.table or 24000

    (
        context.ndb.interfaces.create(ifname=ifname, kind='dummy', state='up')
        .add_ip('%s/24' % ipaddr)
        .commit()
    )

    (
        context.ndb.routes.create(
            table=table, dst='default', gateway=router
        ).commit()
    )

    (context.ndb.routes.create(dst=net, dst_len=24, gateway=router).commit())

    assert route_exists(context.netns, gateway=router, table=table)
    assert context.ndb.routes['default']  # !!! the system must have this
    assert context.ndb.routes[{'dst': 'default', 'table': table}]
    assert context.ndb.routes['%s/24' % net]
    assert context.ndb.routes[{'dst': net, 'dst_len': 24}]


@pytest.mark.parametrize('context', test_matrix, indirect=True)
def test_multipath_ipv4(context):

    ifname = context.new_ifname
    ifaddr = context.new_ipaddr
    hop1 = context.new_ipaddr
    hop2 = context.new_ipaddr
    ipnet = str(context.ipnets[1].network)

    (
        context.ndb.interfaces.create(ifname=ifname, kind='dummy', state='up')
        .ipaddr.create(address=ifaddr, prefixlen=24)
        .commit()
    )

    spec = {
        'dst_len': 24,
        'dst': ipnet,
        'multipath': [{'gateway': hop1}, {'gateway': hop2}],
    }

    if context.table:
        spec['table'] = context.table

    (context.ndb.routes.create(**spec).commit())

    def match_multipath(msg):
        if msg.get_attr('RTA_DST') != ipnet:
            return False
        gws_match = set((hop1, hop2))
        mp = msg.get_attr('RTA_MULTIPATH')
        if mp is None:
            return False
        gws_msg = set([x.get_attr('RTA_GATEWAY') for x in mp])
        return gws_match == gws_msg

    assert address_exists(context.netns, ifname=ifname, address=ifaddr)
    assert route_exists(context.netns, match=match_multipath)


@pytest.mark.parametrize('context', test_matrix, indirect=True)
def test_update_set(context):
    ifaddr = context.new_ipaddr
    router1 = context.new_ipaddr
    router2 = context.new_ipaddr
    ifname = context.new_ifname
    network = str(context.ipnets[1].network)

    (
        context.ndb.interfaces.create(ifname=ifname, kind='dummy', state='up')
        .ipaddr.create(address=ifaddr, prefixlen=24)
        .commit()
    )

    spec = {'dst_len': 24, 'dst': network, 'gateway': router1}

    if context.table:
        spec['table'] = context.table

    r = context.ndb.routes.create(**spec).commit()

    assert address_exists(context.netns, ifname=ifname, address=ifaddr)
    assert route_exists(context.netns, dst=network, gateway=router1)

    r.set('gateway', router2).commit()

    assert route_exists(context.netns, dst=network, gateway=router2)


@pytest.mark.parametrize('context', test_matrix, indirect=True)
def test_update_replace(context):
    ifaddr = context.new_ipaddr
    router = context.new_ipaddr
    ifname = context.new_ifname
    network = str(context.ipnets[1].network)

    (
        context.ndb.interfaces.create(ifname=ifname, kind='dummy', state='up')
        .ipaddr.create(address=ifaddr, prefixlen=24)
        .commit()
    )

    spec = {'dst_len': 24, 'dst': network, 'priority': 10, 'gateway': router}

    if context.table:
        spec['table'] = context.table

    (context.ndb.routes.create(**spec).commit())

    assert address_exists(context.netns, ifname=ifname, address=ifaddr)
    assert route_exists(context.netns, dst=network, priority=10)


@pytest.mark.parametrize('context', test_matrix, indirect=True)
def test_same_multipath(context):
    ifaddr = context.new_ipaddr
    gateway1 = context.new_ipaddr
    gateway2 = context.new_ipaddr
    ifname = context.new_ifname
    ipnet1 = str(context.ipnets[1].network)
    ipnet2 = str(context.ipnets[2].network)

    (
        context.ndb.interfaces.create(ifname=ifname, kind='dummy', state='up')
        .add_ip({'address': ifaddr, 'prefixlen': 24})
        .commit()
    )

    # first route with these gateways
    (
        context.ndb.routes.create(
            dst=ipnet1,
            dst_len=24,
            multipath=[{'gateway': gateway1}, {'gateway': gateway2}],
        ).commit()
    )

    # second route with these gateways
    (
        context.ndb.routes.create(
            dst=ipnet2,
            dst_len=24,
            multipath=[{'gateway': gateway1}, {'gateway': gateway2}],
        ).commit()
    )

    def match_multipath(msg):
        if msg.get_attr('RTA_DST') != ipnet2:
            return False
        gws_match = set((gateway1, gateway2))
        mp = msg.get_attr('RTA_MULTIPATH')
        if mp is None:
            return False
        gws_msg = set([x.get_attr('RTA_GATEWAY') for x in mp])
        return gws_match == gws_msg

    assert address_exists(context.netns, ifname=ifname, address=ifaddr)
    assert route_exists(context.netns, match=match_multipath)


@pytest.mark.parametrize('context', test_matrix, indirect=True)
def test_same_metrics(context):
    ifaddr = context.new_ipaddr
    gateway1 = context.new_ipaddr
    gateway2 = context.new_ipaddr
    ifname = context.new_ifname
    ipnet1 = str(context.ipnets[1].network)
    ipnet2 = str(context.ipnets[2].network)
    target = 1300

    (
        context.ndb.interfaces.create(ifname=ifname, kind='dummy', state='up')
        .add_ip({'address': ifaddr, 'prefixlen': 24})
        .commit()
    )

    # first route with these metrics
    (
        context.ndb.routes.create(
            dst=ipnet1, dst_len=24, gateway=gateway1, metrics={'mtu': target}
        ).commit()
    )

    # second route with these metrics
    (
        context.ndb.routes.create(
            dst=ipnet2, dst_len=24, gateway=gateway2, metrics={'mtu': target}
        ).commit()
    )

    # at this point it's already ok - otherwise the test
    # would explode on the second routes.create()
    # but lets double check

    def match_metrics(msg):
        if msg.get_attr('RTA_GATEWAY') != gateway2:
            return False
        mtu = msg.get_attr('RTA_METRICS', rtmsg()).get_attr('RTAX_MTU', 0)
        return mtu == target

    assert address_exists(context.netns, ifname=ifname, address=ifaddr)
    assert route_exists(context.netns, match=match_metrics)


def _test_metrics_update(context, method):

    ifaddr = context.new_ipaddr
    gateway1 = context.new_ipaddr
    ifname = context.new_ifname
    ipnet = str(context.ipnets[1].network)
    target = 1300
    (
        context.ndb.interfaces.create(ifname=ifname, kind='dummy', state='up')
        .ipaddr.create(address=ifaddr, prefixlen=24)
        .commit()
    )

    spec = {
        'dst_len': 24,
        'dst': ipnet,
        'gateway': gateway1,
        'metrics': {'mtu': target},
    }

    if context.table:
        spec['table'] = context.table

    (context.ndb.routes.create(**spec).commit())

    def match_metrics(msg):
        if msg.get_attr('RTA_GATEWAY') != gateway1:
            return False
        mtu = msg.get_attr('RTA_METRICS', rtmsg()).get_attr('RTAX_MTU', 0)
        return mtu == target

    assert address_exists(context.netns, ifname=ifname, address=ifaddr)
    assert route_exists(context.netns, match=match_metrics)

    target = 1500
    #
    # referencing the route via full spec instead of a
    # local variable is important here for the test
    # purposes: thus we check if the cache is working
    # properly and by the spec we hit the same object
    # every time
    context.ndb.routes[spec]['metrics']['mtu'] = target
    getattr(context.ndb.routes[spec], method)()

    assert route_exists(context.netns, match=match_metrics)


@pytest.mark.parametrize('context', test_matrix, indirect=True)
def test_metrics_update_apply(context):
    return _test_metrics_update(context, 'apply')


@pytest.mark.parametrize('context', test_matrix, indirect=True)
def test_metrics_update_commit(context):
    return _test_metrics_update(context, 'commit')



from pyroute2.netlink import nla

register_kind = 'vlan'


class vlan(nla):
    prefix = 'IFLA_'

    nla_map = (
        ('IFLA_FOO_UNSPEC', 'none'),
        ('IFLA_FOO_ID', 'uint16'),
        ('IFLA_FOO_FLAGS', 'vlan_flags'),
        ('IFLA_FOO_EGRESS_QOS', 'qos'),
        ('IFLA_FOO_INGRESS_QOS', 'qos'),
        ('IFLA_FOO_PROTOCOL', 'be16'),
    )

    class vlan_flags(nla):
        fields = (('flags', 'I'), ('mask', 'I'))

    class qos(nla):
        nla_map = (
            ('IFLA_VLAN_QOS_UNSPEC', 'none'),
            ('IFLA_VLAN_QOS_MAPPING', 'qos_mapping'),
        )

        class qos_mapping(nla):
            fields = (('from', 'I'), ('to', 'I'))



from pyroute2.common import get_address_family
from pyroute2.netlink.rtnl import (
    RTM_NEWADDR,
    RTM_NEWLINK,
    RTM_NEWNEIGH,
    RTM_NEWROUTE,
)


def test_get_links(context):
    for msg in context.ipr.get_links():
        assert msg['header']['target'] == 'localhost'
        assert msg['header']['type'] == RTM_NEWLINK
        #
        assert msg['index'] > 0
        ifname = msg.get_attr('IFLA_IFNAME')
        assert isinstance(ifname, str)


def test_get_addr(context):
    for msg in context.ipr.get_addr():
        assert msg['header']['target'] == 'localhost'
        assert msg['header']['type'] == RTM_NEWADDR
        #
        addr = msg.get_attr('IFA_ADDRESS')
        assert isinstance(addr, str)
        assert msg['family'] == get_address_family(addr)
        assert 0 <= msg['prefixlen'] <= 128


def test_get_routes(context):
    for msg in context.ipr.get_routes():
        assert msg['header']['target'] == 'localhost'
        assert msg['header']['type'] == RTM_NEWROUTE


def test_get_neighbours(context):
    for msg in context.ipr.get_neighbours():
        assert msg['header']['target'] == 'localhost'
        assert msg['header']['type'] == RTM_NEWNEIGH
        #
        dst = msg.get_attr('NDA_DST')
        lladdr = msg.get_attr('NDA_LLADDR')
        assert msg['family'] == get_address_family(dst)
        assert isinstance(lladdr, str)
        assert len(lladdr.split(':')) == 6



import uuid

import pytest
from pr2test.context_manager import make_test_matrix
from pr2test.marks import require_root
from pr2test.tools import address_exists, interface_exists

from pyroute2.ndb.transaction import (
    CheckProcess,
    CheckProcessException,
    Not,
    PingAddress,
)

pytestmark = [require_root()]


def test_check_process_basic():
    test = CheckProcess('true')
    assert test.return_code is None
    test.commit()
    test.rollback()
    assert test.return_code == 0


def test_check_process_fail():
    test = CheckProcess('false')
    assert test.return_code is None
    with pytest.raises(CheckProcessException):
        test.commit()
    assert test.return_code == 1


def test_check_process_file_not_found():
    test = CheckProcess(str(uuid.uuid4()))
    assert test.return_code is None
    with pytest.raises(FileNotFoundError):
        test.commit()
    assert test.return_code is None


def test_check_process_timeout():
    test = CheckProcess('sleep 10', timeout=1)
    with pytest.raises(CheckProcessException):
        test.commit()


@pytest.mark.parametrize('command', (None, '', -1, ['s1', 's2'], True))
def test_check_process_wrong_command(command):
    with pytest.raises(TypeError):
        CheckProcess(command)


def test_negation():
    test = CheckProcess('false')
    with pytest.raises(CheckProcessException):
        test.commit()
    Not(test).commit()


def test_ping_ok():
    test = PingAddress('127.0.0.1')
    test.commit()


def test_ping_unreachable():
    test = PingAddress('128.0.0.1')
    with pytest.raises(CheckProcessException):
        test.commit()


def test_ping_unknown():
    test = PingAddress(str(uuid.uuid4()))
    with pytest.raises(CheckProcessException):
        test.commit()


test_matrix = make_test_matrix(
    targets=['local', 'netns'], dbs=['sqlite3/:memory:', 'postgres/pr2test']
)


@pytest.mark.parametrize('context', test_matrix, indirect=True)
def test_multiple_interfaces(context):

    ifname1 = context.new_ifname
    ifname2 = context.new_ifname
    ipaddr1 = context.new_ipaddr
    ipaddr2 = context.new_ipaddr

    (
        context.ndb.begin()
        .push(
            context.ndb.interfaces.create(ifname=ifname1, kind='dummy')
            .set(state='up')
            .set(address='00:11:22:aa:aa:aa')
            .add_ip(address=ipaddr1, prefixlen=24),
            context.ndb.interfaces.create(ifname=ifname2, kind='dummy')
            .set(state='up')
            .set(address='00:11:22:bb:bb:bb')
            .add_ip(address=ipaddr2, prefixlen=24),
        )
        .commit()
    )

    assert interface_exists(
        context.netns, ifname=ifname1, address='00:11:22:aa:aa:aa'
    )
    assert interface_exists(
        context.netns, ifname=ifname2, address='00:11:22:bb:bb:bb'
    )
    assert address_exists(context.netns, ifname=ifname1, address=ipaddr1)
    assert address_exists(context.netns, ifname=ifname2, address=ipaddr2)


@pytest.mark.parametrize('context', test_matrix, indirect=True)
def test_check_context_manager(context):
    ifname1 = context.new_ifname
    ifname2 = context.new_ifname
    with context.ndb.begin() as ctx:
        ctx.push(context.ndb.interfaces.create(ifname=ifname1, kind='dummy'))
        ctx.push(context.ndb.interfaces.create(ifname=ifname2, kind='dummy'))
    assert interface_exists(context.netns, ifname=ifname1)
    assert interface_exists(context.netns, ifname=ifname2)


def test_interfaces_ping(context):

    ifname1 = context.new_ifname
    ifname2 = context.new_ifname
    ipaddr1 = context.new_ipaddr
    ipaddr2 = context.new_ipaddr

    with pytest.raises(CheckProcessException):
        PingAddress(ipaddr1).commit()

    with pytest.raises(CheckProcessException):
        PingAddress(ipaddr2).commit()

    (
        context.ndb.begin()
        .push(
            context.ndb.interfaces.create(ifname=ifname1, kind='dummy')
            .set(state='up')
            .add_ip(address=ipaddr1, prefixlen=24),
            context.ndb.interfaces.create(ifname=ifname2, kind='dummy')
            .set(state='up')
            .add_ip(address=ipaddr2, prefixlen=24),
            PingAddress(ipaddr1, log=context.ndb.log.channel('util')),
            PingAddress(ipaddr2, log=context.ndb.log.channel('util')),
        )
        .commit()
    )

    assert interface_exists(ifname=ifname1)
    assert interface_exists(ifname=ifname2)
    assert address_exists(ifname=ifname1, address=ipaddr1)
    assert address_exists(ifname=ifname2, address=ipaddr2)


def test_interfaces_ping_fail(context):

    ifname1 = context.new_ifname
    ifname2 = context.new_ifname
    ipaddr1 = context.new_ipaddr
    ipaddr2 = context.new_ipaddr

    with pytest.raises(CheckProcessException):
        PingAddress(ipaddr1).commit()

    with pytest.raises(CheckProcessException):
        PingAddress(ipaddr2).commit()

    (
        context.ndb.begin()
        .push(
            context.ndb.interfaces.create(
                ifname=ifname1, kind='dummy', state='up'
            ),
            context.ndb.interfaces.create(
                ifname=ifname2, kind='dummy', state='up'
            ),
        )
        .commit()
    )

    assert interface_exists(ifname=ifname1)
    assert interface_exists(ifname=ifname2)
    assert not address_exists(address=ipaddr1)
    assert not address_exists(address=ipaddr2)

    with pytest.raises(CheckProcessException):
        (
            context.ndb.begin()
            .push(
                context.ndb.interfaces[ifname1].add_ip(
                    address=ipaddr1, prefixlen=24
                ),
                PingAddress(ipaddr2, log=context.ndb.log.channel('util')),
            )
            .commit()
        )

    assert not address_exists(address=ipaddr1)
    assert not address_exists(address=ipaddr2)



import os

from pr2test.marks import require_root

from pyroute2 import TaskStats

pytestmark = [require_root()]


def test_basic():
    with TaskStats() as ts:
        ts.bind()
        ret = ts.get_pid_stat(os.getpid())[0]

        pid = ret.get_nested('TASKSTATS_TYPE_AGGR_PID', 'TASKSTATS_TYPE_PID')
        stats = ret.get_nested(
            'TASKSTATS_TYPE_AGGR_PID', 'TASKSTATS_TYPE_STATS'
        )

        assert stats['cpu_count'] > 0
        assert stats['ac_pid'] == pid == os.getpid()
        assert stats['coremem'] > 0
        assert stats['virtmem'] > 0



import os
from contextlib import ExitStack

import pytest

from pyroute2 import GenericNetlinkSocket, TaskStats
from pyroute2.netlink import nlmsg


def test_bind_first():

    with ExitStack() as sockets:
        ts = sockets.enter_context(TaskStats())
        gs = sockets.enter_context(GenericNetlinkSocket())

        with pytest.raises(RuntimeError) as ets:
            ts.get_pid_stat(os.getpid())

        with pytest.raises(RuntimeError) as egs:
            gs.nlm_request(nlmsg(), gs.prid)

        assert ets.value.args == egs.value.args



import pytest
from pr2test.marks import require_root

pytestmark = [require_root()]


def test_vlan_filter_dump(context):

    ifname1 = context.new_ifname
    ifname2 = context.new_ifname
    context.ndb.interfaces.create(
        ifname=ifname1, kind='bridge', state='up'
    ).commit()
    context.ndb.interfaces.create(
        ifname=ifname2, kind='bridge', state='up'
    ).commit()
    assert len(context.ipr.get_vlans()) >= 2
    for name in (ifname1, ifname2):
        assert len(context.ipr.get_vlans(ifname=name)) == 1
        assert (
            context.ipr.get_vlans(ifname=name)[0].get_attr('IFLA_IFNAME')
        ) == name
        assert (
            context.ipr.get_vlans(ifname=name)[0].get_nested(
                'IFLA_AF_SPEC', 'IFLA_BRIDGE_VLAN_INFO'
            )
        )['vid'] == 1


@pytest.mark.parametrize(
    'arg_name,vid_spec,vid',
    (
        ('vlan_info', {'vid': 568}, 568),
        ('af_spec', {'attrs': [['IFLA_BRIDGE_VLAN_INFO', {'vid': 567}]]}, 567),
    ),
)
def _test_vlan_filter_add(context, arg_name, vid_spec, vid):

    ifname_port = context.new_ifname
    ifname_bridge = context.new_ifname
    port = context.ndb.interfaces.create(
        ifname=ifname_port, kind='dummy', state='up'
    ).commit()
    (
        context.ndb.interfaces.create(
            ifname=ifname_bridge, kind='bridge', state='up'
        )
        .add_port(ifname_port)
        .commit()
    )
    assert vid not in context.ndb.vlans
    spec = {'index': port['index'], arg_name: vid_spec}
    context.ipr.vlan_filter('add', **spec)
    assert context.ndb.vlans.wait(vid=vid, timeout=5)
    context.ipr.vlan_filter('del', **spec)
    assert context.ndb.vlans.wait(vid=vid, timeout=5, action='remove')



from unittest import mock


class parse(mock.Mock):
    def __init__(self, *argv, **kwarg):
        super().__init__(*argv, **kwarg)
        self.timestamp = mock.Mock(name='timestamp')
        self.timestamp.return_value = 3313938316



import fcntl
import os
import subprocess

import pytest
from pr2test.marks import require_root

from pyroute2 import NSPopen

pytestmark = [require_root()]


def test_basic(context):
    nsid = context.new_nsname
    # create NS and run a child
    nsp = NSPopen(
        nsid, ['ip', '-o', 'link'], stdout=subprocess.PIPE, flags=os.O_CREAT
    )
    ret = nsp.communicate()[0].decode('utf-8')
    host_links = [x.ifname for x in context.ndb.interfaces]
    netns_links = [
        x.split(':')[1].split('@')[0].strip()
        for x in ret.split('\n')
        if len(x)
    ]
    assert nsp.wait() == nsp.returncode == 0
    assert set(host_links) & set(netns_links) == set(netns_links)
    assert set(netns_links) < set(host_links)
    assert not set(netns_links) > set(host_links)
    nsp.release()


def test_release(context):
    nsid = context.new_nsname
    nsp = NSPopen(nsid, ['true'], flags=os.O_CREAT, stdout=subprocess.PIPE)
    nsp.communicate()
    nsp.wait()
    nsp.release()
    with pytest.raises(RuntimeError):
        assert nsp.returncode


def test_stdio(context):
    nsid = context.new_nsname
    nsp = NSPopen(nsid, ['ip', 'ad'], flags=os.O_CREAT, stdout=subprocess.PIPE)
    output = nsp.stdout.read()
    nsp.release()
    assert output is not None


def test_fcntl(context):
    nsid = context.new_nsname
    nsp = NSPopen(nsid, ['ip', 'ad'], flags=os.O_CREAT, stdout=subprocess.PIPE)
    flags = nsp.stdout.fcntl(fcntl.F_GETFL)
    nsp.release()
    assert flags == 0


def test_api_class(context):
    api_nspopen = set(dir(NSPopen))
    api_popen = set(dir(subprocess.Popen))
    assert api_nspopen & api_popen == api_popen


def test_api_object(context):
    nsid = context.new_nsname
    nsp = NSPopen(nsid, ['true'], flags=os.O_CREAT, stdout=subprocess.PIPE)
    smp = subprocess.Popen(['true'], stdout=subprocess.PIPE)
    nsp.communicate()
    smp.communicate()
    api_nspopen = set(dir(nsp))
    api_popen = set(dir(smp))
    minimal = set(('communicate', 'kill', 'wait'))
    assert minimal & (api_nspopen & api_popen) == minimal
    smp.wait()
    nsp.wait()
    assert nsp.returncode == smp.returncode == 0
    nsp.release()



import socket
import struct

import pytest
from pr2test.marks import require_root

pytestmark = [require_root()]


def test_flush_rules(context):
    ifaddr1 = context.new_ipaddr
    ifaddr2 = context.new_ipaddr
    init = len(context.ipr.get_rules(family=socket.AF_INET))
    assert len(context.ipr.get_rules(priority=lambda x: 100 < x < 500)) == 0
    context.ipr.rule('add', table=10, priority=110)
    context.ipr.rule('add', table=15, priority=150, action='FR_ACT_PROHIBIT')
    context.ipr.rule('add', table=20, priority=200, src=ifaddr1)
    context.ipr.rule('add', table=25, priority=250, dst=ifaddr2)
    assert len(context.ipr.get_rules(priority=lambda x: 100 < x < 500)) == 4
    assert len(context.ipr.get_rules(src=ifaddr1)) == 1
    assert len(context.ipr.get_rules(dst=ifaddr2)) == 1
    context.ipr.flush_rules(
        family=socket.AF_INET, priority=lambda x: 100 < x < 500
    )
    assert len(context.ipr.get_rules(priority=lambda x: 100 < x < 500)) == 0
    assert len(context.ipr.get_rules(src=ifaddr1)) == 0
    assert len(context.ipr.get_rules(dst=ifaddr2)) == 0
    assert len(context.ipr.get_rules(family=socket.AF_INET)) == init


def test_basic(context):
    context.ipr.rule('add', table=10, priority=32000)
    assert (
        len(
            [
                x
                for x in context.ipr.get_rules()
                if x.get_attr('FRA_PRIORITY') == 32000
                and x.get_attr('FRA_TABLE') == 10
            ]
        )
        == 1
    )
    context.ipr.rule('delete', table=10, priority=32000)
    assert (
        len(
            [
                x
                for x in context.ipr.get_rules()
                if x.get_attr('FRA_PRIORITY') == 32000
                and x.get_attr('FRA_TABLE') == 10
            ]
        )
        == 0
    )


def test_fwmark(context):
    context.ipr.rule('add', table=15, priority=32006, fwmark=10)
    assert (
        len(
            [
                x
                for x in context.ipr.get_rules()
                if x.get_attr('FRA_PRIORITY') == 32006
                and x.get_attr('FRA_TABLE') == 15
                and x.get_attr('FRA_FWMARK')
            ]
        )
        == 1
    )
    context.ipr.rule('delete', table=15, priority=32006, fwmark=10)
    assert (
        len(
            [
                x
                for x in context.ipr.get_rules()
                if x.get_attr('FRA_PRIORITY') == 32006
                and x.get_attr('FRA_TABLE') == 15
                and x.get_attr('FRA_FWMARK')
            ]
        )
        == 0
    )


def test_fwmark_mask_normalized(context):
    context.ipr.rule('add', table=15, priority=32006, fwmark=10, fwmask=20)
    assert (
        len(
            [
                x
                for x in context.ipr.get_rules()
                if x.get_attr('FRA_PRIORITY') == 32006
                and x.get_attr('FRA_TABLE') == 15
                and x.get_attr('FRA_FWMARK')
                and x.get_attr('FRA_FWMASK')
            ]
        )
        == 1
    )
    context.ipr.rule('delete', table=15, priority=32006, fwmark=10, fwmask=20)
    assert (
        len(
            [
                x
                for x in context.ipr.get_rules()
                if x.get_attr('FRA_PRIORITY') == 32006
                and x.get_attr('FRA_TABLE') == 15
                and x.get_attr('FRA_FWMARK')
                and x.get_attr('FRA_FWMASK')
            ]
        )
        == 0
    )


def test_fwmark_mask_raw(context):
    context.ipr.rule('add', table=15, priority=32006, fwmark=10, FRA_FWMASK=20)
    assert (
        len(
            [
                x
                for x in context.ipr.get_rules()
                if x.get_attr('FRA_PRIORITY') == 32006
                and x.get_attr('FRA_TABLE') == 15
                and x.get_attr('FRA_FWMARK')
                and x.get_attr('FRA_FWMASK')
            ]
        )
        == 1
    )
    context.ipr.rule(
        'delete', table=15, priority=32006, fwmark=10, FRA_FWMASK=20
    )
    assert (
        len(
            [
                x
                for x in context.ipr.get_rules()
                if x.get_attr('FRA_PRIORITY') == 32006
                and x.get_attr('FRA_TABLE') == 15
                and x.get_attr('FRA_FWMARK')
                and x.get_attr('FRA_FWMASK')
            ]
        )
        == 0
    )


def test_bad_table(context):
    with pytest.raises(struct.error):
        context.ipr.rule('add', table=-1, priority=32000)


def test_big_table(context):
    context.ipr.rule('add', table=1024, priority=32000)
    assert (
        len(
            [
                x
                for x in context.ipr.get_rules()
                if x.get_attr('FRA_PRIORITY') == 32000
                and x.get_attr('FRA_TABLE') == 1024
            ]
        )
        == 1
    )
    context.ipr.rule('delete', table=1024, priority=32000)
    assert (
        len(
            [
                x
                for x in context.ipr.get_rules()
                if x.get_attr('FRA_PRIORITY') == 32000
                and x.get_attr('FRA_TABLE') == 1024
            ]
        )
        == 0
    )


def test_src_dst(context):
    context.ipr.rule(
        'add',
        table=17,
        priority=32005,
        src='10.0.0.0',
        src_len=24,
        dst='10.1.0.0',
        dst_len=24,
    )
    assert (
        len(
            [
                x
                for x in context.ipr.get_rules()
                if x.get_attr('FRA_PRIORITY') == 32005
                and x.get_attr('FRA_TABLE') == 17
                and x.get_attr('FRA_SRC') == '10.0.0.0'
                and x.get_attr('FRA_DST') == '10.1.0.0'
                and x['src_len'] == 24
                and x['dst_len'] == 24
            ]
        )
        == 1
    )
    context.ipr.rule(
        'del',
        table=17,
        priority=32005,
        src='10.0.0.0',
        src_len=24,
        dst='10.1.0.0',
        dst_len=24,
    )
    assert (
        len(
            [
                x
                for x in context.ipr.get_rules()
                if x.get_attr('FRA_PRIORITY') == 32005
                and x.get_attr('FRA_TABLE') == 17
                and x.get_attr('FRA_SRC') == '10.0.0.0'
                and x.get_attr('FRA_DST') == '10.1.0.0'
                and x['src_len'] == 24
                and x['dst_len'] == 24
            ]
        )
        == 0
    )



import getpass

import pytest


def require_root(func=None):
    mark = pytest.mark.skipif(
        getpass.getuser() != 'root', reason='no root access'
    )
    if func:
        return mark(func)
    else:
        return mark



import errno
import time

import pytest
from pr2test.context_manager import make_test_matrix, skip_if_not_supported
from pr2test.marks import require_root

from pyroute2 import NetlinkError

pytestmark = [require_root()]
wait_timeout = 30
test_matrix = make_test_matrix(targets=['local', 'netns'])


@pytest.mark.parametrize('context', test_matrix, indirect=True)
@skip_if_not_supported
def test_addr_add(context):
    index, ifname = context.default_interface
    ipaddr = context.new_ipaddr
    ipr = context.ipr
    ndb = context.ndb

    ipr.addr('add', index=index, address=ipaddr, prefixlen=24)
    ndb.addresses.wait(index=index, address=ipaddr, timeout=wait_timeout)


@pytest.mark.parametrize('context', test_matrix, indirect=True)
@skip_if_not_supported
def test_addr_replace(context):
    index, ifname = context.default_interface
    ipaddr1 = context.new_ipaddr
    ipaddr2 = context.new_ipaddr
    ipr = context.ipr
    ndb = context.ndb

    ipr.addr('add', index=index, address=ipaddr1, prefixlen=24)
    ndb.addresses.wait(index=index, address=ipaddr1, timeout=wait_timeout)
    ipr.addr('replace', index=index, address=ipaddr2, prefixlen=24)
    ndb.addresses.wait(index=index, address=ipaddr2, timeout=wait_timeout)


@pytest.mark.parametrize('context', test_matrix, indirect=True)
@skip_if_not_supported
def test_addr_add_local(context):
    index, ifname = context.default_interface
    ipaddr1 = context.new_ipaddr
    ipaddr2 = context.new_ipaddr
    ipr = context.ipr
    ndb = context.ndb

    ipr.addr('add', index=index, address=ipaddr1, local=ipaddr2, prefixlen=24)
    ndb.addresses.wait(
        index=index, address=ipaddr1, local=ipaddr2, timeout=wait_timeout
    )


@pytest.mark.parametrize('context', test_matrix, indirect=True)
@skip_if_not_supported
def test_addr_add_broadcast(context):
    index, ifname = context.default_interface
    ipaddr1 = context.new_ipaddr
    ipaddr2 = context.new_ipaddr
    ipr = context.ipr
    ndb = context.ndb

    ipr.addr(
        'add', index=index, address=ipaddr1, broadcast=ipaddr2, prefixlen=24
    )
    ndb.addresses.wait(
        index=index, address=ipaddr1, broadcast=ipaddr2, timeout=wait_timeout
    )


@pytest.mark.parametrize('context', test_matrix, indirect=True)
@skip_if_not_supported
def test_addr_add_broadcast_default(context):
    index, ifname = context.default_interface
    ipaddr = context.new_ipaddr
    ipr = context.ipr
    ndb = context.ndb

    ipr.addr('add', index=index, address=ipaddr, broadcast=True, prefixlen=24)
    interface = ndb.addresses.wait(
        index=index, address=ipaddr, timeout=wait_timeout
    )
    assert interface['broadcast'] is not None


@pytest.mark.parametrize('context', test_matrix, indirect=True)
@skip_if_not_supported
def test_addr_filter(context):
    index, ifname = context.default_interface
    ipaddr1 = context.new_ipaddr
    ipaddr2 = context.new_ipaddr
    ipaddrB = context.new_ipaddr
    ipr = context.ipr
    ndb = context.ndb

    ipr.addr(
        'add', index=index, address=ipaddr1, broadcast=ipaddrB, prefixlen=24
    )
    ipr.addr(
        'add', index=index, address=ipaddr2, broadcast=ipaddrB, prefixlen=24
    )
    ndb.addresses.wait(index=index, address=ipaddr1, timeout=wait_timeout)
    ndb.addresses.wait(index=index, address=ipaddr2, timeout=wait_timeout)
    assert len(ipr.get_addr(index=index)) >= 2  # remember link-local IPv6
    assert len(ipr.get_addr(address=ipaddr1)) == 1
    assert len(ipr.get_addr(broadcast=ipaddrB)) == 2
    assert len(ipr.get_addr(match=lambda x: x['index'] == index)) >= 2


@pytest.mark.parametrize('context', test_matrix, indirect=True)
@skip_if_not_supported
def test_addr_flush(context):
    index, ifname = context.default_interface
    addresses = [
        context.new_ipaddr,
        context.new_ipaddr,
        context.new_ipaddr,
        context.new_ipaddr,
    ]
    ipr = context.ipr
    ndb = context.ndb
    counter = 5

    for ipaddr in addresses:
        ipr.addr('add', index=index, address=ipaddr, prefixlen=24)
    for ipaddr in addresses:
        ndb.addresses.wait(index=index, address=ipaddr, timeout=wait_timeout)
    ipr.flush_addr(index=index)
    while counter:
        for ipaddr in tuple(addresses):
            if ipaddr not in ndb.addresses:
                addresses.remove(ipaddr)
        if not addresses:
            break
        time.sleep(1)
        counter -= 1
    else:
        raise Exception()


@pytest.mark.parametrize('context', test_matrix, indirect=True)
def test_fail_no_such_device(context):
    ifaddr = context.new_ipaddr
    index = sorted([i['index'] for i in context.ipr.get_links()])[-1] + 10
    with pytest.raises(NetlinkError) as e:
        context.ipr.addr('add', index=index, address=ifaddr, prefixlen=24)
    assert e.value.code == errno.ENODEV


@pytest.mark.parametrize('context', test_matrix, indirect=True)
def test_symbolic_flags(context):
    ipaddr = context.new_ipaddr
    index, ifname = context.default_interface
    context.ipr.link('set', index=index, state='up')
    context.ipr.addr('add', index=index, address=ipaddr, prefixlen=24)
    addr = [
        x for x in context.ipr.get_addr() if x.get_attr('IFA_LOCAL') == ipaddr
    ][0]
    assert 'IFA_F_PERMANENT' in addr.flags2names(addr['flags'])



import pytest
from pr2test.marks import require_root

from pyroute2 import IPDB

pytestmark = [require_root()]


@pytest.fixture
def ictx(context):
    context.ipdb = IPDB(deprecation_warning=False)
    yield context
    context.ipdb.release()


def test_interface_dummy(ictx):

    ifname = ictx.new_ifname
    ipaddr = ictx.new_ipaddr
    interface = ictx.ipdb.create(ifname=ifname, kind='dummy')
    interface.up()
    interface.add_ip(f'{ipaddr}/24')
    interface.commit()

    ictx.ndb.interfaces.wait(action='add', ifname=ifname, timeout=3)
    ictx.ndb.addresses.wait(action='add', address=ipaddr, timeout=3)
    assert ictx.ndb.interfaces[ifname]['state'] == 'up'
    assert (
        ictx.ndb.addresses.wait(action='add', address=ipaddr, prefixlen=24)[
            'index'
        ]
        == interface['index']
    )

    interface.del_ip(f'{ipaddr}/24')
    interface.commit()

    ictx.ndb.addresses.wait(
        action='remove', address=ipaddr, prefixlen=24, timeout=3
    )


def test_interface_veth(ictx):
    netns = ictx.new_nsname
    ictx.ndb.sources.add(netns=netns)
    v0 = ictx.new_ifname
    v1 = ictx.new_ifname

    veth0 = ictx.ipdb.create(ifname=v0, kind='veth', peer=v1)
    veth0.up()
    veth0.commit()

    veth1 = ictx.ipdb.interfaces[v1]
    veth1['net_ns_fd'] = netns
    veth1.commit()

    ictx.ndb.interfaces.wait(ifname=v0, target='localhost', timeout=3)
    ictx.ndb.interfaces.wait(ifname=v1, target=netns, timeout=3)


def test_interface_bridge(ictx):

    ifname = ictx.new_ifname

    with ictx.ipdb.create(ifname=ifname, kind='bridge') as i:
        i.up()
        i['address'] = '00:11:22:33:44:55'
        i['br_stp_state'] = 1
        i['br_forward_delay'] = 1000

    i = ictx.ndb.interfaces.wait(ifname=ifname, timeout=3)
    assert i['state'] == 'up'
    assert i['address'] == '00:11:22:33:44:55'
    assert i['br_stp_state'] == 1
    assert i['br_forward_delay'] == 1000


def test_route_basic(ictx):

    ipaddr = ictx.new_ipaddr
    gateway = ictx.new_ipaddr
    net = ictx.new_ip4net
    ifname = ictx.default_interface.ifname

    with ictx.ipdb.interfaces[ifname] as i:
        i.up()
        i.add_ip(f'{ipaddr}/24')

    ictx.ipdb.routes.add(
        gateway=gateway, dst=f'{net.network}/{net.netmask}'
    ).commit()



import json
import pickle


def _check(context, loaded):
    names = set([x.get_attr('IFLA_IFNAME') for x in loaded])
    indices = set([x['index'] for x in loaded])
    assert names == {x.ifname for x in context.ndb.interfaces.dump()}
    assert indices == {x.index for x in context.ndb.interfaces.dump()}


def test_pickle(context):
    links = context.ipr.link('dump')
    saved = pickle.dumps(links)
    loaded = pickle.loads(saved)
    _check(context, loaded)


def test_json(context):
    links = context.ipr.link('dump')
    saved = json.dumps([x.dump() for x in links])
    msg_type = type(links[0])
    loaded = [msg_type().load(x) for x in json.loads(saved)]
    _check(context, loaded)


def test_dump(context):
    links = context.ipr.link('dump')
    saved = [(type(x), x.dump()) for x in links]
    loaded = [x[0]().load(x[1]) for x in saved]
    _check(context, loaded)



import collections
from unittest import mock


class Client(mock.MagicMock):
    def __init__(self, *argv, **kwarg):
        super().__init__(*argv, **kwarg)
        self.SendPacket = mock.Mock(name='SendPacket')
        self.SendPacket.return_value = collections.namedtuple(
            'reply', ['code']
        )(True)



import pytest
from pr2test.context_manager import make_test_matrix
from pr2test.marks import require_root
from pr2test.tools import interface_exists
from utils import require_user

from pyroute2 import NDB

pytestmark = [require_root()]
test_matrix = make_test_matrix(
    targets=['local', 'netns'], dbs=['sqlite3/:memory:', 'postgres/pr2test']
)


def test_multiple_sources(context):
    '''
    NDB should work with multiple netlink sources

    Check that it actually works:
    * with multiple sources of different kind
    * without the default "localhost" RTNL source
    '''
    nsname = context.new_nsname

    #
    # NB: no 'localhost' record -- important !
    sources = [
        {'target': 'localhost0', 'kind': 'local'},
        {'target': 'localhost1', 'kind': 'netns', 'netns': nsname},
        {'target': 'localhost2', 'kind': 'local'},
    ]
    ndb = None
    #
    # check that all the view has length > 0
    # that means that the sources are working
    with NDB(sources=sources) as ndb:
        assert len(list(ndb.interfaces.dump()))
        assert len(list(ndb.neighbours.dump()))
        assert len(list(ndb.addresses.dump()))
        assert len(list(ndb.routes.dump()))
    # here NDB() gets closed
    #

    #
    # the `ndb` variable still references the closed
    # NDB() object from the code block above, check
    # that all the sources are closed too
    for source in ndb.sources:
        assert ndb.sources[source].nl.closed


@pytest.mark.parametrize('context', test_matrix, indirect=True)
def test_source_localhost_restart(context):
    '''
    The database must be operational after a complete
    restart of any source.
    '''
    require_user('root')
    ifname1 = context.new_ifname
    ifname2 = context.new_ifname
    ndb = context.ndb

    #
    # check that there are existing interfaces
    # loaded into the DB
    assert len(list(ndb.interfaces.dump()))
    #
    # create a dummy interface to prove the
    # source working
    (ndb.interfaces.create(ifname=ifname1, kind='dummy', state='up').commit())
    #
    # an external check
    assert interface_exists(context.netns, ifname=ifname1, state='up')
    #
    # internal checks
    assert ifname1 in ndb.interfaces
    assert ndb.interfaces[ifname1]['state'] == 'up'
    #
    # now restart the source
    # the reason should be visible in the log
    ndb.sources['localhost'].restart(reason='test')
    #
    # the interface must be in the DB (after the
    # source restart)
    assert ifname1 in ndb.interfaces
    #
    # create another one
    (
        ndb.interfaces.create(
            ifname=ifname2, kind='dummy', state='down'
        ).commit()
    )
    #
    # check the interface both externally and internally
    assert interface_exists(context.netns, ifname=ifname2, state='down')
    assert ifname2 in ndb.interfaces
    assert ndb.interfaces[ifname2]['state'] == 'down'
    #
    # cleanup
    ndb.interfaces[ifname1].remove().commit()
    ndb.interfaces[ifname2].remove().commit()
    #
    # check
    assert not interface_exists(ifname=ifname1)
    assert not interface_exists(ifname=ifname2)


@pytest.mark.parametrize('context', test_matrix, indirect=True)
def test_source_netns_restart(context):
    '''
    Netns sources should be operational after restart as well
    '''
    require_user('root')
    nsname = context.new_nsname
    #
    # simple `context.new_ifname` returns ifname only for the main
    # netns, if we want to register the name in a netns, we should
    # use `context.register(netns=...)`
    ifname = context.register(netns=nsname)
    ndb = context.ndb

    #
    # add a netns source, the netns will be created automatically
    ndb.sources.add(netns=nsname)
    #
    # check the interfaces from the netns are loaded into the DB
    assert len(list(ndb.interfaces.dump().filter(target=nsname)))
    #
    # restart the DB
    ndb.sources[nsname].restart(reason='test')
    #
    # check the netns interfaces again
    assert len(list(ndb.interfaces.dump().filter(target=nsname)))
    #
    # create an interface in the netns
    (
        ndb.interfaces.create(
            target=nsname, ifname=ifname, kind='dummy', state='up'
        ).commit()
    )
    #
    # check the interface
    assert interface_exists(nsname, ifname=ifname)
    assert (
        ndb.interfaces[{'target': nsname, 'ifname': ifname}]['state'] == 'up'
    )
    #
    # netns will be remove automatically by the fixture as well
    # as interfaces inside the netns


def count_interfaces(ndb, target):
    return (
        ndb.schema.fetchone(
            '''
                      SELECT count(*) FROM interfaces
                      WHERE
                          f_target = '%s' AND
                          f_index != 0
                      '''
            % target
        )
    )[0]


@pytest.mark.parametrize('context', test_matrix, indirect=True)
def test_disconnect_localhost(context):
    '''
    Disconnecting the `localhost` source should not break the DB
    '''
    require_user('root')
    nsname = context.new_nsname
    localhost_ifnum = 0
    nsname_ifnum = 0
    total_ifnum = 0

    #
    # attach the NetNS source
    context.ndb.sources.add(netns=nsname)

    #
    # lock the DB
    with context.ndb.schema.readonly:
        total_ifnum = context.ndb.interfaces.dump().count()
        localhost_ifnum = (
            context.ndb.interfaces.dump().filter(target='localhost').count()
        )
        nsname_ifnum = (
            context.ndb.interfaces.dump().filter(target=nsname).count()
        )

        assert localhost_ifnum == count_interfaces(context.ndb, 'localhost')
        assert nsname_ifnum == count_interfaces(context.ndb, nsname)
        assert 0 < count_interfaces(context.ndb, 'localhost') < total_ifnum
        assert 0 < count_interfaces(context.ndb, nsname) < total_ifnum

    context.ndb.sources.remove('localhost')

    with context.ndb.schema.readonly:
        #
        # the number of 'localhost' interfaces must be 0 here
        s = len(list(context.ndb.interfaces.dump().filter(target='localhost')))
        assert s == 0
        assert count_interfaces(context.ndb, 'localhost') == 0
        #
        # the number of `nsname` interfaces must remain the same as before
        s = len(list(context.ndb.interfaces.dump().filter(target=nsname)))
        assert s > 0
        assert count_interfaces(context.ndb, nsname) == s



from socket import AF_UNIX

from pr2test.marks import require_root

from pyroute2 import DiagSocket

pytestmark = [require_root()]


def test_basic():
    sstats_set = set()
    pstats_set = set()
    sstats = None
    fd = None

    with DiagSocket() as ds:
        ds.bind()
        sstats = ds.get_sock_stats(family=AF_UNIX)
        for s in sstats:
            sstats_set.add(s['udiag_ino'])

    with open('/proc/net/unix') as fd:
        for line in fd.readlines():
            line = line.split()
            try:
                pstats_set.add(int(line[6]))
            except ValueError:
                pass

    assert sstats_set == pstats_set



from pr2test.marks import require_root
from pr2test.tools import address_exists, interface_exists, route_exists

pytestmark = [require_root()]


def test_cm_interface_create(context):
    '''
    Create an interface using context manager syntax
    '''
    ifname = context.new_ifname
    with context.ndb.interfaces.create(
        ifname=ifname, kind='dummy', state='down'
    ):
        pass
    assert interface_exists(context.netns, ifname=ifname, state='down')
    return ifname


def test_cm_address_create(context):
    '''
    Create an address using context manager syntax
    '''
    ifname = test_cm_interface_create(context)
    ipaddr = context.new_ipaddr
    with context.ndb.addresses.create(
        index=context.ndb.interfaces[ifname]['index'],
        address=ipaddr,
        prefixlen=24,
    ):
        pass
    assert address_exists(context.netns, ifname=ifname, address=ipaddr)


def test_cm_interface_change_assign(context):
    '''
    ::
        with interface as i:
            i['state'] = 'up'
    '''
    ifname = test_cm_interface_create(context)
    with context.ndb.interfaces[ifname] as i:
        i['state'] = 'up'
    assert interface_exists(context.netns, ifname=ifname, state='up')


def test_cm_interface_change_set_argv(context):
    '''
    ::
        with interface as i:
            i.set('state', 'up')
    '''
    ifname = test_cm_interface_create(context)
    with context.ndb.interfaces[ifname] as i:
        i.set('state', 'up')
    assert interface_exists(context.netns, ifname=ifname, state='up')


def test_cm_interface_change_set_kwarg(context):
    '''
    ::
        with interface as i:
            i.set(state='up')
    '''
    ifname = test_cm_interface_create(context)
    with context.ndb.interfaces[ifname] as i:
        i.set(state='up')
    assert interface_exists(context.netns, ifname=ifname, state='up')


def test_routes_spec_dst_len(context):

    ipaddr = context.new_ipaddr
    gateway = context.new_ipaddr
    ifname = context.new_ifname
    ipnet = str(context.ipnets[1].network)
    table = 24000

    (
        context.ndb.interfaces.create(ifname=ifname, kind='dummy', state='up')
        .add_ip(address=ipaddr, prefixlen=24)
        .commit()
    )

    (
        context.ndb.routes.create(
            dst=ipnet, dst_len=24, gateway=gateway, table=table
        ).commit()
    )

    assert route_exists(context.netns, dst=ipnet, table=table)
    r1 = context.ndb.routes.get('%s/24' % ipnet)
    r2 = context.ndb.routes.get({'dst': '%s/24' % ipnet})
    r3 = context.ndb.routes.get({'dst': ipnet, 'dst_len': 24})
    r4 = context.ndb.routes['%s/24' % ipnet]
    r5 = context.ndb.routes[{'dst': '%s/24' % ipnet}]
    r6 = context.ndb.routes[{'dst': ipnet, 'dst_len': 24}]
    assert r1 == r2 == r3 == r4 == r5 == r6


def test_string_key_in_interfaces(context):

    ifname = context.new_ifname
    address = '00:11:22:33:44:55'
    f_ifname = context.new_ifname
    f_address = '00:11:22:33:66:66'

    (
        context.ndb.interfaces.create(
            ifname=ifname, kind='dummy', state='up', address=address
        ).commit()
    )

    assert ifname in context.ndb.interfaces
    assert address in context.ndb.interfaces
    assert f_ifname not in context.ndb.interfaces
    assert f_address not in context.ndb.interfaces


def test_string_key_in_addresses(context):

    ifname = context.new_ifname
    ipaddr = context.new_ipaddr
    f_ipaddr = context.new_ipaddr

    (
        context.ndb.interfaces.create(ifname=ifname, kind='dummy', state='up')
        .add_ip(address=ipaddr, prefixlen=24)
        .commit()
    )

    assert ipaddr in context.ndb.addresses
    assert '%s/%i' % (ipaddr, 24) in context.ndb.addresses
    assert f_ipaddr not in context.ndb.addresses
    assert '%s/%i' % (f_ipaddr, 24) not in context.ndb.addresses
    assert '%s/%i' % (ipaddr, 16) not in context.ndb.addresses
    assert '%s/%i' % (ipaddr, 28) not in context.ndb.addresses
    assert '%s/%i' % (ipaddr, 32) not in context.ndb.addresses



#!/usr/bin/python
'''
Usage::

    ./decoder.py [module] [data_file]

Sample::

    ./decoder.py pyroute2.netlink.rtnl.tcmsg.tcmsg ./sample_packet_01.data
    ./decoder.py pyroute2.netlink.nl80211.nl80211cmd ./nl80211.data

Module is a name within rtnl hierarchy. File should be a
binary data in the escaped string format (see samples).
'''
import sys
from importlib import import_module
from pprint import pprint

from pyroute2.common import hexdump, load_dump

mod = sys.argv[1]
mod = mod.replace('/', '.')
f = open(sys.argv[2], 'r')
s = mod.split('.')
package = '.'.join(s[:-1])
module = s[-1]
m = import_module(package)
met = getattr(m, module)


data = load_dump(f)

offset = 0
inbox = []
while offset < len(data):
    msg = met(data[offset:])
    msg.decode()
    print(hexdump(msg.data))
    pprint(msg)
    print('.' * 40)
    offset += msg['header']['length']



# required by iw_scan_rsp.dump
import datetime  # noqa: F401
import json

import pytest

from pyroute2.common import load_dump
from pyroute2.netlink.nl80211 import MarshalNl80211
from pyroute2.netlink.rtnl.iprsocket import MarshalRtnl


@pytest.mark.parametrize(
    'sample,marshal',
    (
        ('test_unit/test_nlmsg/addrmsg_ipv4.dump', MarshalRtnl),
        ('test_unit/test_nlmsg/gre_01.dump', MarshalRtnl),
        ('test_unit/test_nlmsg/iw_info_rsp.dump', MarshalNl80211),
        ('test_unit/test_nlmsg/iw_scan_rsp.dump', MarshalNl80211),
    ),
)
def test_marshal(sample, marshal):
    parsed = []
    with open(sample, 'r') as buf:
        meta = {}
        parsed = marshal().parse(load_dump(buf, meta))
        if 'application/json' in meta:
            messages = json.loads(meta['application/json'])
        elif 'application/x-python-code' in meta:
            messages = eval(meta['application/x-python-code'])
        else:
            raise KeyError('sample messages not found')
    assert len(parsed) == len(messages)
    for parsed_msg, sample_value in zip(parsed, messages):
        sample_msg = type(parsed_msg)()
        sample_msg.setvalue(sample_value)
        assert parsed_msg == sample_msg



#!/usr/bin/env python
import subprocess

version_module = "pyroute2/config/version.py"
version_output_file = "VERSION"
version_input_file = "VERSION"


def get_project_version():
    """
    Get the project version

    1. fetch version from git
    2. if not available, fallback to the version file in the repo
    """
    try:
        version = subprocess.check_output(
            ("git", "describe"), stderr=subprocess.DEVNULL
        ).decode("utf-8")
    except (FileNotFoundError, subprocess.CalledProcessError):
        with open(version_input_file, "r") as f:
            version = f.read()

    version = version.strip().split("-")

    if len(version) > 1:
        version = "{version[0]}.post{version[1]}".format(**locals())
    else:
        version = version[0]
    return version


if __name__ == "__main__":
    version = get_project_version()
    with open(version_module, "w") as f:
        f.write('__version__ = "%s"\n' % version)
    with open(version_output_file, "w") as f:
        f.write("%s\n" % version)



import socket

import pytest

from pyroute2 import IPRoute
from pyroute2.netlink import nlmsg


def test_context_manager():
    with IPRoute() as ipr:
        ipr.get_links()


def test_multiple_instances():
    ipr1 = IPRoute()
    ipr2 = IPRoute()
    ipr1.close()
    ipr2.close()


def test_close():
    ipr = IPRoute()
    ipr.get_links()
    ipr.close()
    # Shouldn't be able to use the socket after closing
    with pytest.raises(socket.error):
        ipr.get_links()


def test_fileno():
    ipr1 = IPRoute()
    ipr2 = IPRoute(fileno=ipr1.fileno())

    ipr1.close()
    with pytest.raises(OSError) as e:
        ipr2.get_links()
    assert e.value.errno == 9  # sendto -> Bad file descriptor

    with pytest.raises(OSError) as e:
        ipr2.close()
    assert e.value.errno == 9  # close -> Bad file descriptor


def test_get_policy_map(context):
    assert isinstance(context.ipr.get_policy_map(), dict)


def test_register_policy(context):
    context.ipr.register_policy(100, nlmsg)
    context.ipr.register_policy({101: nlmsg})
    context.ipr.register_policy(102, nlmsg)

    assert context.ipr.get_policy_map()[100] == nlmsg
    assert context.ipr.get_policy_map(101)[101] == nlmsg
    assert context.ipr.get_policy_map([102])[102] == nlmsg

    context.ipr.unregister_policy(100)
    context.ipr.unregister_policy([101])
    context.ipr.unregister_policy({102: nlmsg})

    assert 100 not in context.ipr.get_policy_map()
    assert 101 not in context.ipr.get_policy_map()
    assert 102 not in context.ipr.get_policy_map()



from socket import AF_INET

import pytest
from pr2test.context_manager import make_test_matrix, skip_if_not_supported
from pr2test.marks import require_root

from pyroute2.common import AF_MPLS

pytestmark = [require_root()]

test_matrix = make_test_matrix(dbs=['sqlite3/:memory:', 'postgres/pr2test'])


def get_mpls_routes(context):
    return len(tuple(context.ndb.routes.getmany({'family': AF_MPLS})))


@pytest.mark.parametrize('context', test_matrix, indirect=True)
@skip_if_not_supported
def test_via_ipv4(context):

    ifname = context.new_ifname
    ifaddr = context.new_ipaddr
    router = context.new_ipaddr

    l1 = get_mpls_routes(context)

    i = (
        context.ndb.interfaces.create(ifname=ifname, kind='dummy', state='up')
        .add_ip('%s/24' % (ifaddr,))
        .commit()
    )

    rt_spec = {
        'family': AF_MPLS,
        'oif': i['index'],
        'via': {'family': AF_INET, 'addr': router},
        'newdst': {'label': 0x20},
    }

    rt = context.ndb.routes.create(**rt_spec).commit()

    l2 = get_mpls_routes(context)
    assert l2 > l1
    rt.remove().commit()
    l3 = get_mpls_routes(context)
    assert l3 < l2
    assert rt.state == 'invalid'


@pytest.mark.parametrize('context', test_matrix, indirect=True)
@skip_if_not_supported
def test_encap_mpls(context):

    ifname = context.new_ifname
    ifaddr = context.new_ipaddr
    gateway = context.new_ipaddr
    ipnet = str(context.ipnets[1].network)

    (
        context.ndb.interfaces.create(ifname=ifname, kind='dummy', state='up')
        .add_ip('%s/24' % (ifaddr,))
        .commit()
    )

    rt_spec = {
        'dst': '%s/24' % ipnet,
        'gateway': gateway,
        'encap': {'type': 'mpls', 'labels': [20, 30]},
    }
    (context.ndb.routes.create(**rt_spec).commit())



from pr2test import custom_link_kind
from pr2test.custom_link_kind.foo import vlan as foo_vlan
from pr2test.marks import require_root

from pyroute2 import NetlinkError

pytestmark = [require_root()]


def test_register_fail(context):
    ifname = context.new_ifname

    try:
        context.ipr.link(
            'add',
            ifname=ifname,
            link=context.default_interface[0],
            kind='vlan',
            foo_id=101,
        )
    except NetlinkError as e:
        if e.code == 22:  # Invalid argument
            return
    raise Exception('test failed')


def test_register_path(context):
    ifname = context.new_ifname

    old = context.ipr.list_link_kind()['vlan']
    context.ipr.register_link_kind(path='test_linux/pr2test/custom_link_kind/')
    context.ipr.link(
        'add',
        ifname=ifname,
        link=context.default_interface[0],
        kind='vlan',
        foo_id=101,
    )
    assert (
        context.ipr.link('get', ifname=ifname)[0].get_nested(
            'IFLA_LINKINFO', 'IFLA_INFO_DATA', 'IFLA_FOO_ID'
        )
        == 101
    )
    context.ipr.register_link_kind(module={'vlan': old})


def test_register_pkg(context):
    ifname = context.new_ifname

    old = context.ipr.list_link_kind()['vlan']
    context.ipr.register_link_kind(pkg=custom_link_kind)
    context.ipr.link(
        'add',
        ifname=ifname,
        link=context.default_interface[0],
        kind='vlan',
        foo_id=101,
    )
    assert (
        context.ipr.link('get', ifname=ifname)[0].get_nested(
            'IFLA_LINKINFO', 'IFLA_INFO_DATA', 'IFLA_FOO_ID'
        )
        == 101
    )
    context.ipr.register_link_kind(module={'vlan': old})


def test_register_module(context):
    ifname = context.new_ifname

    old = context.ipr.list_link_kind()['vlan']
    context.ipr.register_link_kind(module={'vlan': foo_vlan})
    context.ipr.link(
        'add',
        ifname=ifname,
        link=context.default_interface[0],
        kind='vlan',
        foo_id=101,
    )
    assert (
        context.ipr.link('get', ifname=ifname)[0].get_nested(
            'IFLA_LINKINFO', 'IFLA_INFO_DATA', 'IFLA_FOO_ID'
        )
        == 101
    )
    context.ipr.register_link_kind(module={'vlan': old})



import os
import pathlib
import sys

import pytest


def get_examples(*argv):
    root = pathlib.Path(os.environ['WORKSPACE'])
    examples = [
        fname
        for fname in root.joinpath(*argv).iterdir()
        if not fname.name.endswith('.swp')
    ]
    return {
        'argnames': 'example',
        'argvalues': examples,
        'ids': [x.name for x in examples],
    }


@pytest.mark.parametrize(**get_examples('examples', 'lab'))
def test_block(example, pytester):
    os.chdir(example.as_posix())
    result = pytester.run(sys.executable, 'check.py')
    assert result.ret == 0



import errno
import socket
from time import sleep
from uuid import uuid4

import pytest
from pr2test.marks import require_root

from pyroute2.ipset import IPSet, IPSetError, PortEntry, PortRange
from pyroute2.netlink.exceptions import NetlinkError
from pyroute2.netlink.nfnetlink.ipset import (
    IPSET_ERR_TYPE_SPECIFIC,
    IPSET_FLAG_WITH_FORCEADD,
)

pytestmark = [require_root()]


def parse_ip(entry):
    ip_from = entry.get_attr("IPSET_ATTR_IP_FROM")
    return ip_from.get_attr("IPSET_ATTR_IPADDR_IPV4")


def parse_net(entry):
    net = parse_ip(entry)
    cidr = entry.get_attr("IPSET_ATTR_CIDR")
    if cidr is not None:
        net += "/{0}".format(cidr)
    return net


def ipset_type_to_entry_type(ipset_type):
    return ipset_type.split(":", 1)[1].split(",")


def list_ipset(ipset_name):
    with IPSet() as sock:
        res = {}
        msg_list = sock.list(ipset_name)
        adt = "IPSET_ATTR_ADT"
        proto = "IPSET_ATTR_DATA"
        stype = "IPSET_ATTR_TYPENAME"
        for msg in msg_list:
            for x in msg.get_attr(adt).get_attrs(proto):
                entry = ""
                msg_stypes = msg.get_attr(stype)
                if msg_stypes is None:
                    msg_stypes = "hash:ip"
                for st in ipset_type_to_entry_type(msg_stypes):
                    if st == "ip":
                        entry = parse_ip(x)
                    elif st == "net":
                        entry = parse_net(x)
                    elif st == "iface":
                        entry += x.get_attr("IPSET_ATTR_IFACE")
                    elif st == "set":
                        entry += x.get_attr("IPSET_ATTR_NAME")
                    entry += ","

                entry = entry.strip(",")

                res[entry] = (
                    x.get_attr("IPSET_ATTR_PACKETS"),
                    x.get_attr("IPSET_ATTR_BYTES"),
                    x.get_attr("IPSET_ATTR_COMMENT"),
                    x.get_attr("IPSET_ATTR_TIMEOUT"),
                    x.get_attr("IPSET_ATTR_SKBMARK"),
                    x.get_attr("IPSET_ATTR_SKBPRIO"),
                    x.get_attr("IPSET_ATTR_SKBQUEUE"),
                )
        return res


def ipset_exists(ipset_name):
    with IPSet() as sock:
        try:
            sock.headers(ipset_name)
            return True
        except IPSetError as e:
            if e.code == errno.ENOENT:
                return False
            raise


def test_create_exclusive_fail(ipset, ipset_name):
    ipset.create(ipset_name)
    assert ipset_exists(ipset_name)
    try:
        ipset.create(ipset_name)
    except NetlinkError as e:
        if e.code != errno.EEXIST:
            raise


def test_create_exclusive_success(ipset, ipset_name):
    ipset.create(ipset_name)
    assert ipset_exists(ipset_name)
    ipset.create(ipset_name, exclusive=False)  # do not fail


def test_add_exclusive_fail(ipset, ipset_name):
    ipaddr = "172.16.202.202"
    ipset.create(ipset_name)
    ipset.add(ipset_name, ipaddr)
    assert ipaddr in list_ipset(ipset_name)
    try:
        ipset.add(ipset_name, ipaddr)
    except NetlinkError:
        pass


def test_add_exclusive_success(ipset, ipset_name):
    ipaddr = "172.16.202.202"
    ipset.create(ipset_name)
    ipset.add(ipset_name, ipaddr)
    assert ipaddr in list_ipset(ipset_name)
    ipset.add(ipset_name, ipaddr, exclusive=False)


def test_create_destroy(ipset, ipset_name):
    # create ipset
    ipset.create(ipset_name)
    # assert it exists
    assert ipset_exists(ipset_name)
    # remove ipset
    ipset.destroy(ipset_name)
    # assert it is removed
    assert not ipset_exists(ipset_name)


def test_add_delete(ipset, ipset_name):
    ipaddr = "192.168.1.1"
    # create ipset
    ipset.create(ipset_name)
    assert ipset_exists(ipset_name)
    # add an entry
    ipset.add(ipset_name, ipaddr)
    # check it
    assert ipaddr in list_ipset(ipset_name)
    # delete an entry
    ipset.delete(ipset_name, ipaddr)
    # check it
    assert ipaddr not in list_ipset(ipset_name)


def test_swap(ipset):
    ipset_name_a = str(uuid4())[:16]
    ipset_name_b = str(uuid4())[:16]
    ipaddr_a = "192.168.1.1"
    ipaddr_b = "10.0.0.1"

    # create sets
    ipset.create(ipset_name_a)
    ipset.create(ipset_name_b)
    # add ips
    ipset.add(ipset_name_a, ipaddr_a)
    ipset.add(ipset_name_b, ipaddr_b)
    assert ipaddr_a in list_ipset(ipset_name_a)
    assert ipaddr_b in list_ipset(ipset_name_b)
    # swap sets
    ipset.swap(ipset_name_a, ipset_name_b)
    assert ipaddr_a in list_ipset(ipset_name_b)
    assert ipaddr_b in list_ipset(ipset_name_a)
    # remove sets
    ipset.destroy(ipset_name_a)
    ipset.destroy(ipset_name_b)


def test_counters(ipset, ipset_name):
    ipaddr = "172.16.202.202"
    ipset.create(ipset_name, counters=True)
    ipset.add(ipset_name, ipaddr)
    assert ipaddr in list_ipset(ipset_name)
    assert list_ipset(ipset_name)[ipaddr][0] == 0  # Bytes
    assert list_ipset(ipset_name)[ipaddr][1] == 0  # Packets
    ipset.destroy(ipset_name)

    ipset.create(ipset_name, counters=False)
    ipset.add(ipset_name, ipaddr)
    assert ipaddr in list_ipset(ipset_name)
    assert list_ipset(ipset_name)[ipaddr][0] is None
    assert list_ipset(ipset_name)[ipaddr][1] is None


def test_comments(ipset, ipset_name):
    ipaddr = "172.16.202.202"
    comment = "a very simple comment"
    ipset.create(ipset_name, comment=True)
    ipset.add(ipset_name, ipaddr, comment=comment)
    assert ipaddr in list_ipset(ipset_name)
    assert list_ipset(ipset_name)[ipaddr][2] == comment


def test_skbmark(ipset, ipset_name):
    ipaddr = "172.16.202.202"
    skbmark = (0x100, 0xFFFFFFFF)
    ipset.create(ipset_name, skbinfo=True)
    ipset.add(ipset_name, ipaddr, skbmark=skbmark)
    assert ipaddr in list_ipset(ipset_name)
    assert list_ipset(ipset_name)[ipaddr][4] == skbmark


def test_skbprio(ipset, ipset_name):
    ipaddr = "172.16.202.202"
    skbprio = (1, 10)
    ipset.create(ipset_name, skbinfo=True)
    ipset.add(ipset_name, ipaddr, skbprio=skbprio)
    assert ipaddr in list_ipset(ipset_name)
    assert list_ipset(ipset_name)[ipaddr][5] == skbprio


def test_skbqueue(ipset, ipset_name):
    ipaddr = "172.16.202.202"
    skbqueue = 1
    ipset.create(ipset_name, skbinfo=True)
    ipset.add(ipset_name, ipaddr, skbqueue=skbqueue)
    assert ipaddr in list_ipset(ipset_name)
    assert list_ipset(ipset_name)[ipaddr][6] == skbqueue


def test_maxelem(ipset, ipset_name):
    ipset.create(ipset_name, maxelem=1)
    data = ipset.list(ipset_name)[0].get_attr("IPSET_ATTR_DATA")
    maxelem = data.get_attr("IPSET_ATTR_MAXELEM")
    assert maxelem == 1


def test_hashsize(ipset, ipset_name):
    min_size = 64
    ipset.create(ipset_name, hashsize=min_size)
    data = ipset.list(ipset_name)[0].get_attr("IPSET_ATTR_DATA")
    hashsize = data.get_attr("IPSET_ATTR_HASHSIZE")
    assert hashsize == min_size


def test_forceadd(ipset, ipset_name):
    ipset.create(ipset_name, forceadd=True)
    res = ipset.list(ipset_name)[0].get_attr("IPSET_ATTR_DATA")
    flags = res.get_attr("IPSET_ATTR_CADT_FLAGS")
    assert flags & IPSET_FLAG_WITH_FORCEADD


def test_flush(ipset, ipset_name):
    ipset.create(ipset_name)
    ip_a = "1.1.1.1"
    ip_b = "1.2.3.4"
    ipset.add(ipset_name, ip_a)
    ipset.add(ipset_name, ip_b)
    assert ip_a in list_ipset(ipset_name)
    assert ip_b in list_ipset(ipset_name)
    ipset.flush(ipset_name)
    assert ip_a not in list_ipset(ipset_name)
    assert ip_b not in list_ipset(ipset_name)


def test_rename(ipset, ipset_name):
    ipset_name_bis = str(uuid4())[:16]
    ipset.create(ipset_name)
    ipset.rename(ipset_name, ipset_name_bis)
    assert ipset_exists(ipset_name_bis)
    assert not ipset_exists(ipset_name)


def test_timeout(ipset, ipset_name):
    ip = "1.2.3.4"
    ipset.create(ipset_name, timeout=1)
    ipset.add(ipset_name, ip)
    sleep(2)
    assert ip not in list_ipset(ipset_name)
    # check that we can overwrite default timeout value
    ipset.add(ipset_name, ip, timeout=5)
    sleep(2)
    assert ip in list_ipset(ipset_name)
    assert list_ipset(ipset_name)[ip][3] > 0  # timeout
    sleep(3)
    assert ip not in list_ipset(ipset_name)


def test_net_and_iface_stypes(ipset, ipset_name):
    test_values = (
        ("hash:net", ("192.168.1.0/31", "192.168.12.0/24")),
        ("hash:net,iface", ("192.168.1.0/24,eth0", "192.168.2.0/24,wlan0")),
    )
    for stype, test_values in test_values:
        ipset.create(ipset_name, stype=stype)
        etype = stype.split(":", 1)[1]
        assert ipset_exists(ipset_name)
        for entry in test_values:
            ipset.add(ipset_name, entry, etype=etype)
            assert entry in list_ipset(ipset_name)
            ipset.delete(ipset_name, entry, etype=etype)
            assert entry not in list_ipset(ipset_name)
        ipset.destroy(ipset_name)
        assert not ipset_exists(ipset_name)


def test_tuple_support(ipset, ipset_name):
    test_values = (
        (
            "hash:net,iface",
            (("192.168.1.0/24", "eth0"), ("192.168.2.0/24", "wlan0")),
        ),
    )
    for stype, test_values in test_values:
        ipset.create(ipset_name, stype=stype)
        etype = stype.split(":", 1)[1]
        assert ipset_exists(ipset_name)
        for entry in test_values:
            ipset.add(ipset_name, entry, etype=etype)
            assert ipset.test(ipset_name, entry, etype=etype)
            ipset.delete(ipset_name, entry, etype=etype)
            assert not ipset.test(ipset_name, entry, etype=etype)
        ipset.destroy(ipset_name)


def test_net_with_dash(ipset, ipset_name):
    stype = "hash:net"
    ipset.create(ipset_name, stype=stype)
    # The kernel will split this kind of strings to subnets
    ipset.add(ipset_name, "192.168.1.0-192.168.1.33", etype="net")
    assert "192.168.1.0/27" in list_ipset(ipset_name)
    assert "192.168.1.32/31" in list_ipset(ipset_name)


def test_double_net(ipset, ipset_name):
    stype = "hash:net,port,net"
    etype = "net,port,net"
    ipset.create(ipset_name, stype=stype)
    port = PortEntry(80, protocol=socket.getprotobyname("tcp"))

    ipset.add(
        ipset_name, ("192.168.0.0/24", port, "192.168.2.0/24"), etype=etype
    )


def test_custom_hash_values(ipset, ipset_name):
    stype = "hash:net"
    maxelem = 16384
    hashsize = 64
    ipset.create(ipset_name, stype=stype, maxelem=maxelem, hashsize=hashsize)

    res = ipset.list(ipset_name)[0].get_attr("IPSET_ATTR_DATA")

    assert res.get_attr("IPSET_ATTR_HASHSIZE") == hashsize
    assert res.get_attr("IPSET_ATTR_MAXELEM") == maxelem
    assert res.get_attr("IPSET_ATTR_REFERENCES") == 0


def test_list_set(ipset, ipset_name):
    subname = str(uuid4())[:16]
    subtype = "hash:net"

    ipset.create(subname, stype=subtype)
    ipset.create(ipset_name, "list:set")

    ipset.add(ipset_name, subname, etype="set")
    assert subname in list_ipset(ipset_name)
    assert ipset.test(ipset_name, subname, etype="set")

    res = ipset.list(subname)[0].get_attr("IPSET_ATTR_DATA")
    assert res.get_attr("IPSET_ATTR_REFERENCES") == 1

    ipset.delete(ipset_name, subname, etype="set")
    assert subname not in list_ipset(ipset_name)
    ipset.destroy(subname)


def test_bitmap_port(ipset, ipset_name):
    ipset_type = "bitmap:port"
    etype = "port"
    port_range = (1000, 6000)

    ipset.create(ipset_name, stype=ipset_type, bitmap_ports_range=port_range)
    ipset.add(ipset_name, 1002, etype=etype)
    assert ipset.test(ipset_name, 1002, etype=etype)

    add_range = PortRange(2000, 3000, protocol=None)
    ipset.add(ipset_name, add_range, etype=etype)
    assert ipset.test(ipset_name, 2001, etype=etype)
    assert ipset.test(ipset_name, 3000, etype=etype)
    assert not ipset.test(ipset_name, 4000, etype=etype)

    # Check that delete is working as well
    ipset.delete(ipset_name, add_range, etype=etype)
    assert not ipset.test(ipset_name, 2001, etype=etype)

    # Test PortEntry without protocol set
    port_entry = PortEntry(2001)
    ipset.add(ipset_name, port_entry, etype=etype)
    try:
        ipset.add(ipset_name, 18, etype=etype)
        assert False
    except NetlinkError as e:
        assert e.code == IPSET_ERR_TYPE_SPECIFIC


def test_port_range_with_proto(ipset, ipset_name):
    ipset_type = "hash:net,port"
    etype = "net,port"
    port_range = PortRange(1000, 2000, protocol=socket.IPPROTO_UDP)
    port_entry = PortEntry(1001, protocol=socket.IPPROTO_UDP)

    ipset.create(ipset_name, stype=ipset_type)
    ipset.add(ipset_name, ("192.0.2.0/24", port_range), etype=etype)

    assert ipset.test(ipset_name, ("192.0.2.0/24", port_range), etype=etype)
    assert ipset.test(ipset_name, ("192.0.2.2/32", port_entry), etype=etype)
    # change protocol, that should not be in
    port_range.protocol = socket.IPPROTO_TCP
    assert not ipset.test(
        ipset_name, ("192.0.2.0/24", port_range), etype="net,port"
    )
    port_entry.port = 2
    assert not ipset.test(
        ipset_name, ("192.0.2.0/24", port_entry), etype="net,port"
    )

    # same example than in ipset man pages
    proto = socket.getprotobyname("vrrp")
    port_entry.port = 0
    port_entry.protocol = proto
    ipset.add(ipset_name, ("192.0.2.0/24", port_entry), etype=etype)
    ipset.test(ipset_name, ("192.0.2.0/24", port_entry), etype=etype)


def test_set_by(ipset, ipset_name):
    old_vers = ipset._proto_version

    # check revision supported by kernel
    msg = ipset.get_proto_version()
    version = msg[0].get_attr("IPSET_ATTR_PROTOCOL")
    if version < 7:
        pytest.skip("Kernel does not support this feature")

    # set version
    ipset._proto_version = 7
    # create set
    ipset.create(ipset_name)
    # get index
    msg = ipset.get_set_byname(ipset_name)
    idx = msg[0].get_attr("IPSET_ATTR_INDEX")
    # get set name by index
    msg = ipset.get_set_byindex(idx)
    name_found = msg[0].get_attr("IPSET_ATTR_SETNAME")
    # restore version back to original
    ipset._proto_version = old_vers
    assert ipset_name == name_found
